/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "BoxOfficeAppDelegate.h"
#import "cocos2d.h"
#import "MenuScene.h"


#include <mach/host_info.h>
#include <mach/mach_init.h>
#include <mach/mach_host.h>

@implementation BoxOfficeAppDelegate

@synthesize window;

- (void) applicationDidFinishLaunching:(UIApplication*)application
{
	// Init the window
	window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
	
	// cocos2d will inherit these values
	[window setUserInteractionEnabled:YES];	
	[window setMultipleTouchEnabled:YES];
	
	// Try to use CADisplayLink director
	// if it fails (SDK < 3.1) use Threaded director
	if( ! [CCDirector setDirectorType:CCDirectorTypeDisplayLink] )
		[CCDirector setDirectorType:CCDirectorTypeDefault];
	
	// Use RGBA_8888 buffers
	// Default is: RGB_565 buffers
	[[CCDirector sharedDirector] setPixelFormat:kPixelFormatRGBA8888];
	
	// Create a depth buffer of 16 bits
	// Enable it if you are going to use 3D transitions or 3d objects
//	[[Director sharedDirector] setDepthBufferFormat:kDepthBuffer16];
	
	// Default texture format for PNG/BMP/TIFF/JPEG/GIF images
	// It can be RGBA8888, RGBA4444, RGB5_A1, RGB565
	// You can change anytime.
	[CCTexture2D setDefaultAlphaPixelFormat:kTexture2DPixelFormat_RGBA8888];

	// before creating any layer, set the landscape mode
//	[[Director sharedDirector] setDeviceOrientation:CCDeviceOrientationLandscapeLeft];
	[[CCDirector sharedDirector] setAnimationInterval:1.0/60];
//	[[Director sharedDirector] setDisplayFPS:YES];
	
	// create an openGL view inside a window
	[[CCDirector sharedDirector] attachInView:window];	
	[window makeKeyAndVisible];		

	// for iPad, but this works on iPhone also.
	[[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationChanged:)
												 name:@"UIDeviceOrientationDidChangeNotification" object:nil];
	
	// loadAudio

	if(iPhone_iPad == iPhone){
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Collision"				File:@"Collision_iPhone"					Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Explosion"				File:@"BoxExplosion_iPhone"					Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"TOB"						File:@"SameBoxCollision_iPhone"				Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Factory"					File:@"LauncherSlide1_iPhone"				Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Factory2"					File:@"LauncherSlide2_iPhone"				Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"GameOver"					File:@"GameOver_iPhone"						Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"GameOverWarning"			File:@"GameOverWarning_iPhone"				Ext:@"m4a" Loop:true];
	}else{
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Collision"				File:@"Collision"						Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Explosion"				File:@"BoxExplosion"					Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"TOB"						File:@"SameBoxCollision"				Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Factory"					File:@"LauncherSlide1"					Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Factory2"					File:@"LauncherSlide2"					Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"GameOver"					File:@"GameOver"						Ext:@"m4a" Loop:false];
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"GameOverWarning"			File:@"GameOverWarning"					Ext:@"m4a" Loop:true];
	}

	m_gameManager = [[GameManager alloc] init];
	[[CCDirector sharedDirector] runWithScene: [[MenuScene node] initWithGameManager:m_gameManager]];
}


- (void)applicationWillResignActive:(UIApplication *)application {
	[[CCDirector sharedDirector] pause];
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
	[[CCDirector sharedDirector] resume];
}

- (void)applicationDidReceiveMemoryWarning:(UIApplication *)application {
	[[CCTextureCache sharedTextureCache] removeUnusedTextures];
}

- (void)applicationWillTerminate:(UIApplication *)application {
	[[CCDirector sharedDirector] end];
}

- (void)applicationSignificantTimeChange:(UIApplication *)application {
	[[CCDirector sharedDirector] setNextDeltaTimeZero:YES];
}

// for iPad
-(void) orientationChanged:(NSNotification *)notification
{
	UIDeviceOrientation orientation = [[UIDevice currentDevice] orientation];
	if (orientation == UIDeviceOrientationPortrait || orientation == UIDeviceOrientationPortraitUpsideDown)
	{
		[[CCDirector sharedDirector] setDeviceOrientation:(ccDeviceOrientation)orientation];
	}
}

- (void)dealloc {
	[[CCDirector sharedDirector] release];
	[window release];
	[m_gameManager release];
	[super dealloc];
}

@end
