/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "CreditScene.h"

#define CREDIT_FONT_SIZE 18
#define CREDIT_HEIGHT 25

@implementation CreditScene

-(id) initWithGameManager: (GameManager *) gameManager{
	self = [super init];
	if( self != nil )
	{
		CGSize s = [[CCDirector sharedDirector] winSize];
		CCSprite* bg;
		
		if(iPhone_iPad == iPhone){
			bg = [CCSprite spriteWithFile:@"Credit_Background_iPhone.jpg"];
		}else{
			bg = [CCSprite spriteWithFile:@"Credit_Background.jpg"];
		}
		
		[bg setPosition:cpv(s.width/2, s.height/2)];
		[self addChild:bg z:0];
		[self addChild: [[CreditLayer node] initWithGameManager:gameManager] z:1];
	}
	
	return self;	
}

-(void) dealloc
{
	[[CCTextureCache sharedTextureCache] removeUnusedTextures];
	
	[super dealloc];
}
@end


@implementation CreditLayer

@synthesize m_touch_y, m_touch_time;

-(id) initWithGameManager: (GameManager *) gameManager{
	self = [super init];
	if( self != nil )
	{
		m_gameManager = gameManager;
		
		isTouchEnabled = YES;
		ScrollStop = FALSE;
		startPositionLock = FALSE;

		// Touch Detect
		self.m_touch_y		=	[[NSMutableArray alloc] init];			// touch log
		self.m_touch_time	=	[[NSMutableArray alloc] init];			// touch log (msec)
		
		credits = [[NSMutableArray alloc] initWithObjects:
				   @"- GAME DESIGN -",
				   @"- PROGRAMMING -",
				   @"- GRAPHIC DESIGN -",
				   @" ",
				   @"AAA",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @"- IMAGE -",
				   @" ",
				   @"BBB",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @"- MUSIC -",
				   @" ",
				   @"CCC",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @"Powered By cocos2D for iPhone",			// DO NOT REMOVE THIS CREDIT
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @"BOX OFFICES for iPhone/iPad",				// DO NOT REMOVE THIS CREDIT
				   @"The Original Game is",						// DO NOT REMOVE THIS CREDIT
				   @"Designed by Entatonic.net",				// DO NOT REMOVE THIS CREDIT
				   @" ",
				   @"Copyright (c) 2010 Entatonic.net",			// DO NOT REMOVE THIS CREDIT
				   @"http://entatonic.net",						// DO NOT REMOVE THIS CREDIT
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @" ",
				   @"Produced By DDD.net",
				   nil
				   ];

		m_tag_num = 0;
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = [CCLabel labelWithString:@"APP NAME"					// ******* REPLACE WITH YOUR APP NAME **********
										 fontName:@"AmericanTypewriter" fontSize:CREDIT_FONT_SIZE * 2  * iPhone_iPad];
			[label setColor:ccc3(5+200*(1-i) ,5+200*(1-i) ,5+200*(1-i))];
			[label setPosition: cpv(0,-100)];	// out of display
			[self addChild:label z:0 tag:m_tag_num];
			m_tag_num++;
		}

		for(NSString *credit in credits){
			CCLabel* label = [CCLabel labelWithString:credit
										 fontName:@"AmericanTypewriter" fontSize:CREDIT_FONT_SIZE  * iPhone_iPad];
			[label setColor:ccc3(5, 5, 5)];
			[label setPosition: cpv(0,-100)];	// out of display
			[self addChild:label z:0 tag:m_tag_num];
			m_tag_num++;
		}
		
		
		CGSize s = [[CCDirector sharedDirector] winSize];
		offset = -s.height - CREDIT_FONT_SIZE * iPhone_iPad;

		maxOffset = offset + 
					CREDIT_FONT_SIZE * iPhone_iPad +					// "App Name" size / 2
					4 * CREDIT_HEIGHT  * iPhone_iPad	+				// Space
					[credits count] * CREDIT_HEIGHT  * iPhone_iPad + 	// Credits
					CREDIT_FONT_SIZE  * iPhone_iPad / 2 +				// Credit on last line
					s.height  / 3;										// place on display size (1/4 bottom side)

		top_y_axis = - CREDIT_FONT_SIZE /* *2 */ * iPhone_iPad /* /2 */;
		bottom_y_axis = -s.height - CREDIT_FONT_SIZE /* *2 */ * iPhone_iPad /* /2 */;
		vy = 0;
	}
	return self;
}

-(void)draw{
	CGSize s = [[CCDirector sharedDirector] winSize];
	
	// Touch Control
	if(ScrollStop == FALSE || outofScroll == TRUE){	 
		if(offset < bottom_y_axis && startPositionLock == FALSE)	{
			vy = (bottom_y_axis - offset) / 2;	if( abs(vy) < 1 ) vy = 0.0;
		}
		if(offset <	top_y_axis && startPositionLock == TRUE)	{
			vy = (top_y_axis - offset) / 2;		if( abs(vy) < 1 ) vy = 0.0;
		}
		if(offset > maxOffset){			// vy only in case of stopping scroll(=touching)
			vy = (maxOffset - offset) / 2;		if( abs(vy) < 1 ) vy = 0.0;
		}
	}
		
	// Scroll Control
	if(vy != 0){
		offset = offset + vy / 2 * iPhone_iPad;
		if(vy > 0) {vy = vy - (float)0.25 * iPhone_iPad * iPhone_iPad; if (vy <= 0) vy = 0;}	
		if(vy < 0) {vy = vy + (float)0.25 * iPhone_iPad * iPhone_iPad; if (vy >= 0) vy = 0;}	
	}else{
		if(ScrollStop == FALSE){	// Auto Scroll
			offset = offset + 2;
			if(offset > maxOffset) offset = maxOffset;
		}
	}

	if(offset > top_y_axis)
		startPositionLock = TRUE;

	
	// Draw
	int y = s.height;
	int x = s.width / 2;

	CCLabel* label = (CCLabel*)[self getChildByTag:0];
	[label setPosition: cpv(x ,y + offset)];
	
	label = (CCLabel*) [self getChildByTag:1];
	[label setPosition: cpv(x + 2, y + 2 + offset)];
	
	for(int i = 2; i < m_tag_num; i++){
		CCLabel* label = (CCLabel*) [self getChildByTag:i];
		[label setPosition: cpv(x, y + offset	- 4 * CREDIT_HEIGHT  * iPhone_iPad
												- (CREDIT_HEIGHT  * iPhone_iPad) * i)];
	}
	
}

-(void) ccTouchesBegan:(NSSet*)touches withEvent:(UIEvent *)event
{
	ScrollStop = TRUE;
	vy = 0;
	
	outofScroll = FALSE;
	if(offset < bottom_y_axis && startPositionLock == FALSE)	{outofScroll=TRUE;}
	if(offset <	top_y_axis && startPositionLock == TRUE)		{outofScroll=TRUE;}
	if(offset > maxOffset)										{outofScroll=TRUE;}
	
	UITouch *touch = [touches anyObject];
	CGPoint touch_point = [touch locationInView: [touch view]];
	touch_point = [[CCDirector sharedDirector] convertToGL:touch_point];
	
	NSTimeInterval start_time = [NSDate timeIntervalSinceReferenceDate];
	[self.m_touch_y addObject:[NSNumber numberWithDouble:touch_point.y]];
	[self.m_touch_time addObject:[NSNumber numberWithDouble:start_time]];

	return;
}

-(void) ccTouchesMoved:(NSSet*)touches withEvent:(UIEvent *)event
{
	UITouch *touch = [touches anyObject];
	CGPoint touch_point = [touch locationInView: [touch view]];
	touch_point = [[CCDirector sharedDirector] convertToGL:touch_point];
	
	if( ((offset > bottom_y_axis && startPositionLock == FALSE)||
		 (offset > top_y_axis  && startPositionLock == TRUE) ||
		 offset <= maxOffset) && outofScroll == FALSE){
		offset = offset +
			(touch_point.y - [[self.m_touch_y objectAtIndex:[self.m_touch_y count]-1] doubleValue]);	
	}
	
	NSTimeInterval moving_time = [NSDate timeIntervalSinceReferenceDate];
	[self.m_touch_y addObject:[NSNumber numberWithDouble:touch_point.y]];
	[self.m_touch_time addObject:[NSNumber numberWithDouble:moving_time]];
	
	return;
}


-(void) ccTouchesEnded:(NSSet*)touches withEvent:(UIEvent*)event
{
	ScrollStop = FALSE;
	
	NSTimeInterval end_time = [NSDate timeIntervalSinceReferenceDate];
	
    UITouch *touch = [touches anyObject];
	CGPoint touch_point = [touch locationInView: [touch view]];
	touch_point = [[CCDirector sharedDirector] convertToGL:touch_point];
	
	for(int i = 0; i < [self.m_touch_time count]; i++){
		double diffY = touch_point.y - [[self.m_touch_y objectAtIndex:i] doubleValue];
		double diff_time = end_time - [[self.m_touch_time objectAtIndex:i] doubleValue];
		double total_diff = touch_point.y - [[self.m_touch_y objectAtIndex:0] doubleValue];
		if(diff_time < 0.1 && diffY*diffY > 25* iPhone_iPad && outofScroll == FALSE){
			vy = diffY;
			if(vy > 25* iPhone_iPad) vy = 25* iPhone_iPad;
			if(vy < -25* iPhone_iPad) vy = -25* iPhone_iPad;
			break;
		}
		if(total_diff * total_diff < 25 * iPhone_iPad){
			CCScene *s = [CCScene node];
			[s addChild: [[MenuLayer node] initWithGameManager:m_gameManager]];
			[[CCDirector sharedDirector] replaceScene: s];
		}
	}
	
	for(int i = [self.m_touch_time count] - 1; i >= 0 ; i--){
		[self.m_touch_y removeObjectAtIndex:i];
		[self.m_touch_time removeObjectAtIndex:i];
	}
	
	return;
}

-(void)dealloc
{
	for(int i = [self.m_touch_time count] - 1; i >= 0 ; i--){
		[self.m_touch_y removeObjectAtIndex:i];
		[self.m_touch_time removeObjectAtIndex:i];
	}
	self.m_touch_y = nil;
	self.m_touch_time = nil;
	
	[credits release];
	[super dealloc];
}


@end
