/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import <Foundation/Foundation.h>
#import "Launcher.h"
#import "Relations.h"
#import "StageObjects.h"
#import "MyOpenAL.h"
#import "BoxExplosion.h"
#import "ScoreManager.h"
#import "StageManager.h"
#import "boxObject.h"

@interface FieldManager : NSObject {
	ScoreManager	*m_scoreManager;
	StageManager	*m_stageManager;
	NSMutableArray			*listOnfield;			// Object list on field
	NSMutableArray			*listToDelete;			// Size 0 Object List to keep the relations
	int		m_layer_counter;				// TENTATIVE, need manager
	int		m_launcher_layer_counter;		// TENTATIVE, need manager

	// Field Cieling
	int		ceiling_y;	
	cpShape * m_shape_ceiling;
	cpBody	*m_body_ceiling;
		
	StageObjects *m_StageObjects;
	Relations *m_Relations;
	Launcher *m_Launcher;
	
	cpSpace *m_space;
	boxObject *boxObj;
	
	// Field Line
	int display_width;
	int display_height;
	int field_x;
	int field_width;
	int field_y;
	int field_height;
	
	// Event & Condition
	BOOL gameLost;
}
-(void) resetField;

// Field Size
-(int) getfield_x;
-(int) getfield_y;
-(int) getfield_width;
-(int) getfield_height;

// Ceiling Control
-(void) moveUpCeiling;
-(void) moveDownCeiling;
-(void) moveCeilingShape;
-(int) getCeilingY;

-(id) initWithScoreManager: (ScoreManager *) scoreManager stageManager:(StageManager*) stageManager;
-(Relations *) getRelations;	// read only

-(int) getNumOfObjectsOnField;

-(void) moveLauncherLeft:(CCLayer *)spriteOwnerCocosLayer stageManager:(StageManager *) stageManager;
-(void) resetLauncher:(CCLayer *) spriteOwnerCocosLayer stageManager:(StageManager *) stageManager;
-(void) placeInitialBoxes:(CCLayer *) spriteOwnerCocosLayer stageManager:(StageManager *) stageManager;
-(boxObject *) createNewObjectOnLauncher:(CCLayer *) spriteOwnerCocosLayer
					boxtype:(int)box_type
					scale:(int) scale
					 imagePath:(NSString *) imagePath
							vx:(float)launching_vx
							vy:(float)launching_vy;

-(boxObject *) createNewObjectAndInstance:(CCLayer *) spriteOwnerCocosLayer	// for replaced object
								  boxtype:(int)box_type
								imagePath:(NSString *) imagePath
									scale:(int) init_scale
										x:(float) init_x
										y:(float) init_y
									   vx:(float) init_vx
									   vy:(float) init_vy
							displayingscale:(int)displayingscale;

-(BOOL) launchBox_vx:(float)launcher_vx vy:(float)launcher_vy;
-(void) stepAnimation:(CCLayer *) spriteOwnerCocosLayer stageHalt:(BOOL) halt friction:(float)friction
						stageManager:(StageManager *) stageManager;
-(boxObject *) RenewSizeOfSpace:(boxObject *) ReNewBox;
-(cpSpace *) getcpSpace;


// Relations
-(BOOL) setRelations:(cpShape *) shapeA withShape:(cpShape *) shapeB;
-(int) getBoxTypeOfListOnFieldBycpShape:(cpShape *)shape;
-(int) getIndexOfListOnFieldBycpShape:(cpShape *)shape;

-(void) removeBoxRelation:(int) index;

-(boxObject *) getBoxObjectFrom_index:(int) index;
-(int) getBoxObject_Count;
-(BOOL) getGameRedAlart;
-(BOOL) getGameLost;

@property (assign) NSMutableArray *listOnfield;
@property (assign) StageObjects *m_StageObjects;
@property (assign) Relations *m_Relations;
@property (assign) Launcher *m_Launcher;
@property (assign) ScoreManager	*m_scoreManager;
@property (assign) StageManager	*m_stageManager;
@property (assign) cpSpace *m_space;

@end
