/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "FieldManager.h"

@implementation FieldManager

@synthesize listOnfield;

-(id) initWithScoreManager: (ScoreManager *) scoreManager stageManager:(StageManager*) stageManager
{
	self = [super init];
	if( self != nil )
	{
		m_scoreManager = scoreManager;
		m_stageManager = stageManager;
		
		listOnfield =  [[NSMutableArray alloc] init];	// filed objects initialize
		listToDelete = [[NSMutableArray alloc] init];	// filed objects to Delete

		//Initialize Chipmunk:
		cpInitChipmunk();
		
		m_space = cpSpaceNew();

		[self resetField];

		CGSize s = [[CCDirector sharedDirector] winSize];
		int x_width = 4 * iPhone_iPad;
		int y_width = 4 * iPhone_iPad;
		display_width = s.width;
		display_height = s.height;
		field_x = x_width;
		field_y = y_width + 30 * iPhone_iPad;
		field_width = s.width - field_x * 2;
		field_height = s.height - field_y * 2;
		
		gameLost = FALSE;
		
		// For random seed (Box Type)
		srand(time(nil));
	}
	
	return self;
}

-(void) resetField{
	// Init Cocos2d
	cpSpaceInit(m_space);
	cpSpaceResizeStaticHash(m_space, 400.0f, 40);
	cpSpaceResizeActiveHash(m_space, 100, 600);
	
	m_space->gravity = cpv(0, 0);
	m_space->elasticIterations = m_space->iterations;
	
	m_StageObjects =	[[StageObjects alloc] initWithcpSpace:m_space];
	m_Relations =		[[Relations alloc] init];
	m_Launcher =		[[Launcher alloc] initWithcpSpace:m_space];
	
	ceiling_y = 0;

	cpVect verts[4];

	cpShape * shape;
	cpBody *staticBody = cpBodyNew(INFINITY, INFINITY);

	// Bottom
	verts[0] = cpv(field_x, -display_height	/* 0 */);
	verts[1] = cpv(field_x, field_y);
	verts[2] = cpv(field_x + field_width,  field_y);
	verts[3] = cpv(field_x + field_width,  -display_height/* 0 */);
	
	shape = cpPolyShapeNew(staticBody, 4, verts, cpvzero);

	shape->e = 1.0; shape->u = 0.0;//1.0;
	shape->collision_type = 0;
	cpSpaceAddStaticShape(m_space, shape);

	// Right Side
	verts[0] = cpv(field_x+field_width, 0);
	verts[1] = cpv(field_x+field_width, field_y + field_height);
	verts[2] = cpv(display_width * 2, field_y + field_height);
	verts[3] = cpv(display_width * 2, 0);
	
	shape = cpPolyShapeNew(staticBody, 4, verts, cpvzero);
	
	shape->e = 1.0; shape->u = 0.0;	//1.0;
	shape->collision_type = 0;
	cpSpaceAddStaticShape(m_space, shape);
	
	// Left Side
	verts[0] = cpv(-display_width,		0);
	verts[1] = cpv(-display_width,		field_y + field_height);
	verts[2] = cpv(field_x,				field_y + field_height);
	verts[3] = cpv(field_x,				0);
	
	shape = cpPolyShapeNew(staticBody, 4, verts, cpvzero);	

	shape->e = 1.0; shape->u = 0.0;	//1.0;
	shape->collision_type = 0;
	cpSpaceAddStaticShape(m_space, shape);

	// Ceiling
	m_body_ceiling = cpBodyNew(INFINITY, INFINITY);

	verts[0] = cpv(field_x,				field_y + field_height);
	verts[1] = cpv(field_x,				display_height * 2);
	verts[2] = cpv(field_x+field_width, display_height * 2);
	verts[3] = cpv(field_x+field_width, field_y + field_height);
	
	m_shape_ceiling = cpPolyShapeNew(m_body_ceiling, 4, verts, cpvzero);	
	
	m_shape_ceiling->e = 1.0; m_shape_ceiling->u = 0.0;//1.0;
	m_shape_ceiling->collision_type = 0;
	cpSpaceAddStaticShape(m_space, m_shape_ceiling);
	
	// Init
	m_layer_counter = 100;
	m_launcher_layer_counter = 160000;
	gameLost = FALSE;
	
	[m_Launcher resetLauncher];

	// Delete all of relations
	for(int i=[m_Relations getCount] -1; i >= 0 ;i--)
		[m_Relations removeBoxRelation:i];
	
	// Delete all of listOnField
	for(int i=[listOnfield count] -1; i >= 0 ;i--){
		[[listOnfield objectAtIndex:i] release];
		[listOnfield removeObjectAtIndex:i];
	}

	// Delete all of listToDelete
	for(int i=[listToDelete count] -1; i >= 0 ;i--){
		[[listToDelete objectAtIndex:i] release];
		[listToDelete removeObjectAtIndex:i];
	}
}

// Field Size
-(int) getfield_x{
	return field_x;
}
-(int) getfield_y{
	return field_y;
}
-(int) getfield_width{
	return field_width;
}
-(int) getfield_height{
	return field_height;
}

-(void) moveUpCeiling{
	if ( ceiling_y > 0 ) ceiling_y = ceiling_y - [m_stageManager getCielingDown] * 6;
	if ( ceiling_y < 0 ) ceiling_y = 0;
	[self moveCeilingShape];
}

-(void) moveDownCeiling{
	
	if ( ceiling_y < field_height) ceiling_y = ceiling_y + [m_stageManager getCielingDown];
	[self moveCeilingShape];
}

-(void) moveCeilingShape{
	cpShape * previous_shape = m_shape_ceiling;
	cpBody * previous_body = m_body_ceiling;
	
	cpVect verts[4];

	m_body_ceiling = cpBodyNew(INFINITY, INFINITY);
	
	verts[0] = cpv(field_x,				field_y + field_height - ceiling_y);
	verts[1] = cpv(field_x,				display_height * 2);
	verts[2] = cpv(field_x+field_width, display_height * 2);
	verts[3] = cpv(field_x+field_width, field_y + field_height - ceiling_y);
	
	m_shape_ceiling = cpPolyShapeNew(m_body_ceiling, 4, verts, cpvzero);
	
	m_shape_ceiling->e = 1.0; m_shape_ceiling->u = 0.0;//1.0;
	m_shape_ceiling->collision_type = 0;
	cpSpaceAddStaticShape(m_space, m_shape_ceiling);

	// remove
	cpSpaceRemoveStaticShape(m_space, previous_shape);
	cpSpaceRemoveBody(m_space, previous_body);
}

-(int) getCeilingY{
	return ceiling_y;
}

-(Relations*) getRelations{
	return m_Relations;
}

-(void) moveLauncherLeft:(CCLayer *)spriteOwnerCocosLayer stageManager:(StageManager *) stageManager{
	NSInteger type = [stageManager get_launcher_NextboxType];
	
	[self createNewObjectOnLauncher:spriteOwnerCocosLayer
				  boxtype:type
					scale:20
				imagePath:[NSString stringWithFormat:[stageManager getBoxImage:type]]
					   vx:0.0 vy:0.0];
	
	[m_Launcher moveLauncherLeft];
}

-(void) resetLauncher:(CCLayer *) spriteOwnerCocosLayer stageManager:(StageManager *) stageManager{
	for(int i = 0; i < 7 + ((iPhone_iPad - 1) * 2); i++){
		NSInteger type = [stageManager get_launcher_NextboxType];
		
		[self createNewObjectOnLauncher:spriteOwnerCocosLayer boxtype:type
					    scale:20 imagePath:[stageManager getBoxImage:type]
						   vx:0.0 vy:0.0];
	}
}

-(void) placeInitialBoxes:(CCLayer *) spriteOwnerCocosLayer stageManager:(StageManager *) stageManager{
	for(int i = 0; i < [stageManager countInitialBoxes]; i++){
		[self createNewObjectAndInstance:spriteOwnerCocosLayer
										   boxtype:[stageManager get_initialBox_type:i]
										 imagePath:[stageManager getBoxImage:[stageManager get_initialBox_type:i]]
											 scale:[stageManager get_initialBox_scale:i]
												 x:(float)[stageManager get_initialBox_x:i] * field_width + field_x
												 y:(float)[stageManager get_initialBox_y:i] * field_height + field_y
												vx:[stageManager get_initialBox_vx:i]
												vy:[stageManager get_initialBox_vy:i]
								   displayingscale:0];
	}

}

// Create new box on launcher
-(boxObject *) createNewObjectOnLauncher:(CCLayer *) spriteOwnerCocosLayer
					boxtype:(int)box_type
					scale:(int) scale
					 imagePath:(NSString *) imagePath
					 vx:(float)launching_vx
					 vy:(float)launching_vy{
	boxObj = [[[boxObject alloc] initWithX:(float)[m_Launcher getXFromIndex:[m_Launcher getNumOfObjects]]
										 y:(float)[m_Launcher getYFromIndex:[m_Launcher getNumOfObjects]]
										   boxtype:(int)box_type
										   layer:m_launcher_layer_counter
										   scale:scale
										 cpSpace:m_space
									   imagePath:imagePath
					 spriteOwnerCocosLayer: spriteOwnerCocosLayer
			    			displayingscale:scale] retain];

	
	[listOnfield addObject:boxObj];
	[m_Launcher addOboxObject:boxObj];
	m_launcher_layer_counter++;	
	
	return boxObj;
}

// Create new box in field
-(boxObject *) createNewObjectAndInstance:(CCLayer *) spriteOwnerCocosLayer
					   boxtype:(int)box_type
					 imagePath:(NSString *) imagePath
							scale:(int) init_scale
							x:(float) init_x
							y:(float) init_y
							vx:(float) init_vx
							vy:(float) init_vy
							displayingscale:(int)displayingscale{

	boxObj = [[[boxObject alloc] initWithX:(float)init_x
										 y:(float)init_y
								   boxtype:(int)box_type
									 layer:m_layer_counter
									 scale:init_scale
								   cpSpace:m_space
								 imagePath:imagePath
					spriteOwnerCocosLayer: spriteOwnerCocosLayer
					displayingscale:displayingscale] retain];
	
	
	[listOnfield addObject:boxObj];
	m_layer_counter++;	
	
	[boxObj createBoxInstance:init_vx vy:init_vy];

	return boxObj;
}

-(BOOL) launchBox_vx:(float)launcher_vx vy:(float)launcher_vy{
	
	BOOL flag = [m_Launcher launchBox_vx:(float)launcher_vx vy:(float)launcher_vy];
	if(flag == TRUE) [self moveDownCeiling];
	
	return flag;
}

-(void) stepAnimation:(CCLayer *) spriteOwnerCocosLayer stageHalt:(BOOL) halt friction:(float)friction
						stageManager:(StageManager *) stageManager// Step Animation each
{
	[m_StageObjects stepAnimation];
	[m_Relations stepAnimation];
	
	if(halt == TRUE) return;
	
	// Launcher
	boxObject* invalidObject = [m_Launcher stepAnimation:(StageManager *) stageManager];
	if(invalidObject != 0){
		// Slided without launching, so need to delete box on launcher
		for(int i=[listOnfield count] -1; i >= 0 ;i--){
			boxObject* box = [listOnfield objectAtIndex:i];
			if(box == invalidObject){
				[listOnfield removeObjectAtIndex:i];
				[box releaseShapes];
				[box release];
			}
		}
		[self moveDownCeiling];
	}
	
	// launcher speed control
	int rate = 100;
	if( [self getNumOfObjectsOnField] == 3 ) rate = 80;
	if( [self getNumOfObjectsOnField] == 2 ) rate = 50;
	if( [self getNumOfObjectsOnField] == 1 ) rate = 40;
	
	[m_Launcher setLauncherExpandSpeedRate:rate];
	
	// Box object
	for(int i=[listOnfield count] -1; i >= 0 ;i--){
		boxObject* box = [listOnfield objectAtIndex:i];
		[box stepAnimation];
		
		// box goes to out of field unexpectedly.
		int boxx = [box getX];
		int boxy = [box getY];
		int buffer = BOX_SIZE_PIXEL * [box getScale] / 100;
		if([box inField] == TRUE && [box getScale] != 0.0 &&
		   (boxx < field_x - buffer||
			boxx > field_width + buffer ||
			boxy < field_y - buffer ||
			boxy > field_y + field_height - ceiling_y + buffer)){

			   // small explosion
			   float particle = 50;	// Limitter
			   CCParticleSystem *emitter = [[BoxExplosion node] initWithTotalParticles:particle];
			   emitter.position = cpv([box getX],[box getY]);
			   [emitter setTexture:[[CCTextureCache sharedTextureCache] addImage:[box getImagePath]]];
			   CCLayer* spriteOwnerCocosLayer = [box getspriteOwnerCocosLayer];
			   [spriteOwnerCocosLayer addChild: emitter z:1900];
			
			   [listToDelete addObject:box];
			   [listOnfield removeObjectAtIndex:i];
		}

		// Brakes by friction
		float boxvx = [box getVX];
		float boxvy = [box getVY];
		float brakes = friction; // 0.0: no friction, 2.0: suitable friction, 4.0: hard friction
		if(boxvx * boxvx + boxvy * boxvy > brakes * brakes){
//			if(abs(boxvx)<1.0){										// Remove comment out to execute on emulator to avoid exception of EXC_ARITHMETIC signal, not tested code on device
//				if(boxvy > 0) boxvy = boxvy - brakes;
//				if(boxvy < 0) boxvy = boxvy + brakes;
//			}else if(abs(boxvy)<1.0){
//				if(boxvx > 0) boxvx = boxvx - brakes;
//				if(boxvx < 0) boxvx = boxvx + brakes;
//			}else{
			float atangent = atan(abs(boxvy)/abs(boxvx));			// ***** Sometimes App stops by exception of EXC_ARITHMETIC signal *****
			if(boxvx > 0) boxvx = boxvx - brakes * cos(atangent);
			if(boxvx < 0) boxvx = boxvx + brakes * cos(atangent);
			if(boxvy > 0) boxvy = boxvy - brakes * sin(atangent);
			if(boxvy < 0) boxvy = boxvy + brakes * sin(atangent);
//			}														// Remove comment out to execute on emulator to avoid exception of EXC_ARITHMETIC signal, not tested code on device
		}

		// Limitter
		if(boxvx * boxvx + boxvy * boxvy > 700 * 700){
//			if(abs(boxvx)<1.0){										// Remove comment out to execute on emulator to avoid exception of EXC_ARITHMETIC signal, not tested code on device
//				if(boxvy > 0) boxvy = boxvy - brakes;
//				if(boxvy < 0) boxvy = boxvy + brakes;
//			}else if(abs(boxvy)<1.0){
//				if(boxvx > 0) boxvx = boxvx - brakes;
//				if(boxvx < 0) boxvx = boxvx + brakes;
//			}else{
			float atangent = atan(abs(boxvy)/abs(boxvx));			// ***** Sometimes App stops by exception of EXC_ARITHMETIC signal *****
			if(boxvx > 0) boxvx = boxvx - 100 * cos(atangent);
			if(boxvx < 0) boxvx = boxvx + 100 * cos(atangent);
			if(boxvy > 0) boxvy = boxvy - 100 * sin(atangent);
			if(boxvy < 0) boxvy = boxvy + 100 * sin(atangent);			
//			}														// Remove comment out to execute on emulator to avoid exception of EXC_ARITHMETIC signal, not tested code on device
		}
		
		[box setVXVY:boxvx vy:boxvy];
	}
	
	// OTHER ANIMATION STEP

	if([m_Launcher expiredLauncher_MoveLauncherToLeft] == TRUE){
		[self moveLauncherLeft:spriteOwnerCocosLayer stageManager:stageManager];
	}
	
	// Explosion
	for(int i=[listOnfield count] -1; i >= 0 ;i--){
		boxObject* box = [listOnfield objectAtIndex:i];
		if([box createdInstance] == TRUE && [box isChangingSize] == FALSE	// The Box is in field
		   && [box getScale] > EXPLODE_SCALE){
			[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"Explosion"];
			[self moveUpCeiling];

			// Explosion
			float particle = ([box getScale]-EXPLODE_SCALE + 10) / 10;
			particle = (int)particle * (int)particle * (int)80;
			if(particle > 900) particle = 900;	// Limitter
			if(iPhone_iPad == iPad) particle = particle * 2 / 3; // For iPad
			CCParticleSystem *emitter = [[BoxExplosion node] initWithTotalParticles:particle];
			emitter.position = cpv([box getX],[box getY]);
			[emitter setTexture:[[CCTextureCache sharedTextureCache] addImage:[box getImagePath]]];
			CCLayer* spriteOwnerCocosLayer = [box getspriteOwnerCocosLayer];
			[spriteOwnerCocosLayer addChild: emitter z:1900];

			// Add Score
			[m_scoreManager addScore:[box getScale] * 10 x:[box getX] y:[box getY]];
			
			[box setBoxScale:0];
			[box setDisplayingScale:0];
			[self RenewSizeOfSpace:box];
			
			[listOnfield removeObjectAtIndex:i];
			[box releaseShapes];
			[box release];
		}
	}

	// Size Replace
	for(int i=[listOnfield count] -1; i >= 0 ;i--){
		boxObject* box = [listOnfield objectAtIndex:i];
		// Size Replace in space
		if([box createdInstance] == TRUE && [box isChangingSize] == FALSE	// The Box is in field
		   && [box getScale] != [box getSpaceScale]){
			[self RenewSizeOfSpace:box];

			// remove
			[listOnfield removeObjectAtIndex:i];
			[box releaseShapes];
			[box release];
		}
	}

	// detect box size 0, and animation completed
	for(int i=[listOnfield count] -1; i >= 0 ;i--){
		boxObject* box = [listOnfield objectAtIndex:i];
		if([box getScale] == 0 && [box getDisplayingScale] == 0){
			[listToDelete addObject:box];
			[listOnfield removeObjectAtIndex:i];
		}
	}
	
	// Release box instance
	for(int i=[listToDelete count] -1; i >= 0 ;i--){
		boxObject* box = [listToDelete objectAtIndex:i];
		if([m_Relations hasRelation:box] == FALSE){
			[listToDelete removeObjectAtIndex:i];
			[box releaseShapes];
			[box release];
		}
	}
	
	// Check game over
	int maxBoxSize = 0;
	for(int i=[listOnfield count] -1; i >= 0 ;i--){
		boxObject* box = [listOnfield objectAtIndex:i];
		if([box getScale] == [box getDisplayingScale]){	// if the box is stable
			int boxSize = [box getScale];
			if(maxBoxSize < boxSize) maxBoxSize = boxSize;
		}
	}
	if(field_height - ceiling_y < BOX_SIZE_PIXEL * maxBoxSize / 100/*%*/ * 0.9 /*Buffer*/){
		gameLost = TRUE;
		[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"GameOver"];
		[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:@"GameOverWarning"];
		[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:[NSString stringWithFormat:@"Game_BGM%d", [m_stageManager getStageRound]]];
	}
}

-(boxObject *) RenewSizeOfSpace:(boxObject *) ReNewBox
{
	float replace_x = [ReNewBox getX];
	float replace_y = [ReNewBox getY];
	float replace_vx = [ReNewBox getVX];
	float replace_vy = [ReNewBox getVY];
	//			float replace_angle = [ReNewBox getAngle];
	int	replace_scale = [ReNewBox getScale];
	//			int replace_layer = [ReNewBox getLayer];
	int replace_boxType = [ReNewBox getBoxType];
	NSString* replace_imagepath = [ReNewBox getImagePath];
	CCLayer* replace_spriteOwnerCocosLayer = [ReNewBox getspriteOwnerCocosLayer];

	boxObject* replacedBox = [self createNewObjectAndInstance:replace_spriteOwnerCocosLayer
													  boxtype:replace_boxType
													imagePath:replace_imagepath
														scale:replace_scale
															x:replace_x
															y:replace_y 
														   vx:replace_vx
														   vy:replace_vy
											  displayingscale:replace_scale];
	return replacedBox;
}

-(cpSpace *) getcpSpace
{
	return m_space;
}

// Relations (box - box)
-(BOOL) setRelations:(cpShape *) shapeA withShape:(cpShape *) shapeB{
	// If types of collided boxes are same
	int boxTypeOfshapeA = [self getBoxTypeOfListOnFieldBycpShape:shapeA];
	int boxTypeOfshapeB = [self getBoxTypeOfListOnFieldBycpShape:shapeB];

	if(boxTypeOfshapeA == -1 || boxTypeOfshapeB == -1) return FALSE;

	if(boxTypeOfshapeA == boxTypeOfshapeB){
		boxObject* objA = [listOnfield objectAtIndex:[self getIndexOfListOnFieldBycpShape:(cpShape *)shapeA]];
		boxObject* objB = [listOnfield objectAtIndex:[self getIndexOfListOnFieldBycpShape:(cpShape *)shapeB]];
		
		// size control
		int size_a = [objA getScale];
		int size_b = [objB getScale];
		
		if(size_a == 0 || size_b == 0) return FALSE;	// ignore if one of the box size is 0
		
		// Collision Detect
		[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"Collision"];

		// Set Relations
		if([m_Relations addBoxFrom:objA To:objB] == TRUE){
			// Add Score
			[m_scoreManager addScore: [objA getScale] x:[objA getX] y:[objA getY]];
			[m_scoreManager addScore: [objB getScale] x:[objB getX] y:[objB getY]];
		}
		
		if(size_a >= size_b){
			size_a = size_a + size_b;
			size_b = 0;
		}
		if(size_a < size_b){
			size_b = size_b + size_a;
			size_a = 0;
		}

		[objA setBoxScale:size_a];
		[objB setBoxScale:size_b];

		return TRUE;
	}else{
		// Collision without TOB
		[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"Collision"];
		return FALSE;
	}
}

-(int) getBoxTypeOfListOnFieldBycpShape:(cpShape *)shape{
	for(int i=0; i < [listOnfield count];i++){
		boxObject* box = [listOnfield objectAtIndex:i];
		if(shape == (cpShape *)[box getShape]){
			return [box getBoxType];
		}
	}
	return -1;
}

-(int) getIndexOfListOnFieldBycpShape:(cpShape *)shape{
	for(int i=0; i < [listOnfield count];i++){
		if(shape == (cpShape *)[[listOnfield objectAtIndex:i] getShape]){
			return i;
		}
	}
	return -1;
}

-(int) getNumOfObjectsOnField{
	return [listOnfield count] - [m_Launcher getNumOfObjects];
}

-(void) removeBoxRelation:(int) index{
	[m_Relations removeBoxRelation:index];
}

-(boxObject *) getBoxObjectFrom_index:(int) index{
	return [listOnfield objectAtIndex:index];
}
-(int) getBoxObject_Count{
	return [listOnfield count];
}

-(BOOL) getGameRedAlart{
	int maxBoxSize = 0;
	for(int i=[listOnfield count] -1; i >= 0 ;i--){
		boxObject* box = [listOnfield objectAtIndex:i];
		int boxSize = [box getScale];
		if(maxBoxSize < boxSize) maxBoxSize = boxSize;
	}
	if(field_height - ceiling_y < BOX_SIZE_PIXEL * maxBoxSize / 100 /*%*/ * 1.3){
		if([[MyOpenAL sharedMyOpenAL] isPlayingSoundWithKey:
			[NSString stringWithFormat:@"GameOverWarning"]] == FALSE && gameLost == FALSE	/* Playing */){
			[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"GameOverWarning"];
		}
		return TRUE;
	}else{
		if([[MyOpenAL sharedMyOpenAL] isPlayingSoundWithKey:
				 [NSString stringWithFormat:@"GameOverWarning"]] == TRUE){
			[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:@"GameOverWarning"];
		}
		return FALSE;
	}
}

-(BOOL) getGameLost{
	return gameLost;
}


-(void) dealloc
{
	[m_StageObjects release];
	[m_Relations release];
	[m_Launcher release];

	[super dealloc];
}

@synthesize m_StageObjects, m_Relations, m_Launcher, m_space;
@synthesize m_scoreManager, m_stageManager;

@end
