/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "GameManager.h"


@implementation GameManager

-(id) init {
	self = [super init];
	if(self != nil){
		m_stageManager = [[StageManager alloc] init];
		[m_stageManager resetStage];
		m_scoreManager = [[ScoreManager alloc] init];
		[m_scoreManager resetScore];
		m_fieldManager = [[FieldManager alloc] initWithScoreManager:m_scoreManager stageManager:m_stageManager];
		
		m_display_memory = FALSE;
	}
	return self;
}

-(void) initStageRoundNumber{
	[m_scoreManager resetScore];
	[m_stageManager resetStage];
}

-(void) continueStage{
	[m_scoreManager resetScore];
	[m_stageManager continueStage];
}


-(void) resetStageforNext:(CCLayer *)spriteOwnerCocosLayer{
	gameClear = FALSE;
	gameLost = FALSE;
	wasFlicked = FALSE;
	gamePause = FALSE;
	hit = FALSE;
	flick_line_vx = 0.0f;
	flick_line_vy = 0.0f;
	flick_count = 0;

	if(m_fieldManager != nil){
		[m_fieldManager release];
		m_fieldManager = [[FieldManager alloc] initWithScoreManager:m_scoreManager stageManager:m_stageManager];
	}
	[m_fieldManager resetField];
	[m_scoreManager resetAddingScore];
	
	// reset Launcher
	[m_fieldManager resetLauncher:spriteOwnerCocosLayer stageManager:m_stageManager];

	// Place stage initial boxes
	[m_fieldManager placeInitialBoxes:spriteOwnerCocosLayer stageManager:m_stageManager];
}

-(void) releaseFieldManager{
	if(m_fieldManager != nil){	
		[m_fieldManager release];
		m_fieldManager = [[FieldManager alloc] initWithScoreManager:m_scoreManager stageManager:m_stageManager];
	}
}

-(ScoreManager *) getScoreManager{
	return m_scoreManager;
}

-(StageManager *) getStageManager{
	return m_stageManager;
}

-(FieldManager *) getFieldManager{
	return m_fieldManager;
}

-(Relations *) getRelations{
	return [m_fieldManager getRelations];
}


-(void) moveLauncherLeft:(CCLayer *)spriteOwnerCocosLayer{
	[m_fieldManager moveLauncherLeft:(CCLayer *)spriteOwnerCocosLayer stageManager:m_stageManager];
}

-(void) resetLauncher:(CCLayer *) spriteOwnerCocosLayer{
	[m_fieldManager resetLauncher:spriteOwnerCocosLayer stageManager:m_stageManager];
}

-(BOOL) launchBox_vx:(float)launcher_vx vy:(float)launcher_vy{
	return [m_fieldManager launchBox_vx:(float)launcher_vx vy:(float)launcher_vy];
}

-(int) getStageRound{
	return [m_stageManager getStageRound];
}
-(int) getLogStageNum{
	return [m_stageManager getLogStageNum];
}
-(int) getLogStageRound{
	return [m_stageManager getLogStageRound];
}

-(int) getStageNum{
	return [m_stageManager getStageNum];
}
-(BOOL) incrementStage{
	return [m_stageManager incrementStage];
}

-(void) stepFieldAnimation:(CCLayer *) spriteOwnerCocosLayer stageHalt:(BOOL) halt
{
	[m_fieldManager stepAnimation:(CCLayer *) spriteOwnerCocosLayer
						stageHalt:halt friction:[m_stageManager getFriction]
						stageManager:m_stageManager];
	[m_scoreManager stepAnimation];
	if(flick_count > 0) flick_count--;

	return;
}

-(cpSpace *) getcpSpace
{
	return [m_fieldManager getcpSpace];
}

-(int) getNumOfObjectsOnField{
	return [m_fieldManager getNumOfObjectsOnField];
}
-(int) getCeilingY{
	return [m_fieldManager getCeilingY];
}

-(void) setFlickLine_vx:(double) vx vy:(double)vy{
	flick_line_vx = vx;
	flick_line_vy = vy;
	flick_count = max_flick_count;
}
-(double) getFlickLine_vx{
	return flick_line_vx;
}
-(double) getFlickLine_vy{
	return flick_line_vy;
}
-(int) getFlickLine_count{
	return flick_count;
}

// Relations
-(void) setRelations:(cpShape *) shapeA withShape:(cpShape *) shapeB{
	BOOL collisionDetect = [m_fieldManager setRelations:shapeA withShape:shapeB];
	[self setHit:collisionDetect];
}

-(void) removeBoxRelation:(int) index{
	[m_fieldManager removeBoxRelation:index];
}


-(void) resetScore{
	[m_scoreManager resetScore];
}
-(void) syncScore{
	[m_scoreManager syncScore];
}
-(int) getScore{
	return [m_scoreManager getScore];
}
-(int) getDisplayingScore{
	return [m_scoreManager getDisplayingScore];
}
-(boxObject *) getBoxObjectFrom_index:(int) index{
	return [m_fieldManager getBoxObjectFrom_index:index];
}

-(int) getBoxObject_Count{
	return [m_fieldManager getBoxObject_Count];
}

-(void) setGameClear{
	gameClear = TRUE;
}
-(BOOL) getGameClear{
	return gameClear;
}
-(BOOL) getGameLost{
	return [m_fieldManager getGameLost];
}
-(BOOL) getGameRedAlart{
	return [m_fieldManager getGameRedAlart];
}

-(void) setwasFlicked:(BOOL) Flicked{
	wasFlicked = Flicked;
}

-(BOOL) getwasFlicked{
	return wasFlicked;
}

-(void) setGamePause:(BOOL) Pause{
	gamePause = Pause;
}

-(BOOL) getGamePause{
	return gamePause;
}



-(void) setHit:(BOOL) Hit{
	hit = Hit;
}

-(BOOL) getHit{
	return hit;
}

-(void) setDisplaMemory:(BOOL) displayMemory{
	m_display_memory = displayMemory;
}
-(BOOL) getDisplaMemory{
	return m_display_memory;
}

-(void) dealloc
{
	[super dealloc];
}

@synthesize m_stageManager;
@synthesize m_fieldManager;

@end
