/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "GameScene.h"
#import "iPhone_iPad_Selector.h"
#import "test.h"

#include <mach/host_info.h>
#include <mach/mach_init.h>
#include <mach/mach_host.h>

static void
eachShape(void *ptr, void* unused)
{
	cpShape *shape = (cpShape*) ptr;
	CCSprite *sprite = shape->data;
	if( sprite ) {
		cpBody *body = shape->body;
		[sprite setPosition: cpv( body->p.x, body->p.y)];
		[sprite setRotation: (float) CC_RADIANS_TO_DEGREES( -body->a )];
	}
}


static int boxCollision(cpShape *a, cpShape *b, cpContact *contacts, int numContacts, cpFloat normal_coef, void *data)
{
	GameLayer *game = (GameLayer*) data;
	[game collision_detect:a withShape:b];

	return 1;
}


// ****************************************************************
@implementation GameScene

-(id) initWithGameManager: (GameManager *) gameManager
{
	self = [super init];
	if( self != nil )
	{
		m_gameManager = gameManager;

		TitleLayer* tl = [[TitleLayer node] initWithGameManager:m_gameManager];
		[self addChild:tl z:1];
		
	}
	return self;
}


-(void) dealloc
{
	[[CCTextureCache sharedTextureCache] removeUnusedTextures];
	[super dealloc];
}

@synthesize m_gameManager;

@end



// ****************************************************************
@implementation TitleLayer

-(id) initWithGameManager: (GameManager *) gameManager{
	self = [super init];
	if(self != nil){
		[[CCTextureCache sharedTextureCache] removeUnusedTextures];
		[gameManager releaseFieldManager];
		
		isTouchEnabled = YES;
		m_gameManager = gameManager;

		// Back Ground Image
		CGSize s = [[CCDirector sharedDirector] winSize];
		CCSprite* Image = [[CCSprite spriteWithFile:[[m_gameManager getStageManager] getBackGournd_image]
						  ] retain];
		[Image setPosition:cpv(s.width/2, s.height/2)];
		[self addChild:Image z:0 tag:0];			
		
		// Loop Label
		int LoopNum = [[m_gameManager getStageManager]  getStageLoop];
		if( LoopNum > 1 && [[m_gameManager getStageManager] IsTutorialStage] == FALSE){
			for(int i = 0; i <= 1; i++){	// Shadow
				CCLabel* label;
				label = [CCLabel labelWithString:[NSString stringWithFormat:@"LOOP %d", LoopNum]
									  fontName:@"Helvetica" fontSize:32 * iPhone_iPad];
				[label setPosition: cpv(s.width/2, s.height * 5 / 8 + i)];
				[self addChild: label z:90];
				if(i == 0){	// Shadow
					[label setColor:ccc3([[m_gameManager getStageManager] getTextShadowColor:0]
								 ,[[m_gameManager getStageManager] getTextShadowColor:1] 
								 ,[[m_gameManager getStageManager] getTextShadowColor:2])];
				}else{
					[label setColor:ccc3([[m_gameManager getStageManager] getStageStringColor:0]*i
								 ,[[m_gameManager getStageManager] getStageStringColor:1]*i
								 ,[[m_gameManager getStageManager] getStageStringColor:2]*i)];
				}
			}
		}
		
		// Stage Label
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label;
				label = [CCLabel labelWithString:
								[NSString stringWithFormat:@"STAGE %d",
								 [m_gameManager getStageRound]]
											 fontName:@"Helvetica" fontSize:32 * iPhone_iPad];
			[self addChild: label z:1 + i];
			if(i == 0){	// Shadow
				[label setColor:ccc3([[m_gameManager getStageManager] getTextShadowColor:0]
							 ,[[m_gameManager getStageManager] getTextShadowColor:1] 
							 ,[[m_gameManager getStageManager] getTextShadowColor:2])];
			}else{
				[label setColor:ccc3([[m_gameManager getStageManager] getStageStringColor:0]*i
						 ,[[m_gameManager getStageManager] getStageStringColor:1]*i
						 ,[[m_gameManager getStageManager] getStageStringColor:2]*i)];
			}
			[label setPosition: cpv(s.width/2 + i, s.height/2 + i)];
		}
		
		countstart_time = [NSDate timeIntervalSinceReferenceDate];
		
		// Play Audio Voice
			if(iPhone_iPad == iPhone){
				[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Stage"
												   File:[NSString stringWithFormat:@"Voice_Stage%d_iPhone",[m_gameManager getStageRound]]
													Ext:@"m4a" Loop:false];
			}else{
				[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Stage"
													   File:[NSString stringWithFormat:@"Voice_Stage%d",[m_gameManager getStageRound]]
														Ext:@"m4a" Loop:false];				
			}
			[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"Stage"];
	
		// Play BGM
		if([[MyOpenAL sharedMyOpenAL] loadedSoundWithKey:
			[NSString stringWithFormat:@"Game_BGM%d",[m_gameManager getStageRound]]] == false){
			// load & play
			[self performSelectorInBackground:@selector(loadAudio) withObject:self];
		}
		
		// ver 1.1 additional feature
		// MEMORY LABEL
		CCLabel* label = [CCLabel labelWithString:[NSString stringWithFormat:@"0"]
										 fontName:@"AmericanTypewriter" fontSize:15 * iPhone_iPad];
		[label setColor:ccc3([[m_gameManager getStageManager] getArtistNameStringColor:0],
							 [[m_gameManager getStageManager] getArtistNameStringColor:1],
							 [[m_gameManager getStageManager] getArtistNameStringColor:2])];
		[label setPosition: cpv(100, -1000) ];
		[self addChild:label z:10 tag:1];
		
		[self schedule: @selector(step:)];
	}
	
	return self;
}

// ver 1.1 additional feature
// draw memory
-(void)draw{
	// MEMORY	
	if([m_gameManager getDisplaMemory] == TRUE){
		struct vm_statistics a_vm_info;
		mach_msg_type_number_t a_count = HOST_VM_INFO_COUNT;
		host_statistics( mach_host_self(), HOST_VM_INFO, (host_info_t)&a_vm_info ,&a_count);
		
		CCLabel* label = (CCLabel*)[self getChildByTag:1];
		[label setString:[NSString stringWithFormat:@"%06f MB", (float)a_vm_info.free_count * vm_page_size / 1000000]];
		CGSize s = [[CCDirector sharedDirector] winSize];
		[label setPosition: cpv(s.width / 5, s.height / 24 ) ];
	}else{
		CCLabel* label = (CCLabel*)[self getChildByTag:1];
		[label setPosition: cpv(100, -1000) ];
	}
}


-(void)loadAudio{
	NSAutoreleasePool* pool;
	pool=[[NSAutoreleasePool alloc]init];

	[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:
	 [NSString stringWithFormat:@"Game_BGM%d",[m_gameManager getStageRound]]
										   File:[[m_gameManager getStageManager] getBgm]
											Ext:@"m4a" Loop:true];

	[pool release];
}

-(void) step: (ccTime) delta
{
	if ([NSDate timeIntervalSinceReferenceDate] - countstart_time > 3.0 &&
		[[MyOpenAL sharedMyOpenAL] isPlayingSoundWithKey:
		 [NSString stringWithFormat:@"Stage"]] == FALSE &&			// Played Stage Voice
		[[MyOpenAL sharedMyOpenAL] loadedSoundWithKey:
		 [NSString stringWithFormat:@"Game_BGM%d",[m_gameManager getStageRound]]] == true	// Loaded BGM
		){
			[self unschedule: @selector(step:)];

			[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:@"Stage"];
			[[MyOpenAL sharedMyOpenAL] releaseSoundWithKey:@"Stage"];
		
			[[MyOpenAL sharedMyOpenAL] playSoundWithKey:
			 [NSString stringWithFormat:@"Game_BGM%d",[m_gameManager getStageRound]]];
		
			CCScene *sc = [CCScene node];
			[sc addChild: [[GameLayer node] initWithGameManager:m_gameManager] z:1000001];
			[[CCDirector sharedDirector] replaceScene: sc];
		}
}

-(void) dealloc
{
	[[CCTextureCache sharedTextureCache] removeUnusedTextures];
	[super dealloc];
}

@end

// ****************************************************************
@implementation GameBackGroundLayer

-(id) initWithGameManager: (GameManager *) gameManager
{
	self = [super init];
	if( self != nil )
	{
		m_gameManager = gameManager;
		
		// Back Ground Image
		CGSize s = [[CCDirector sharedDirector] winSize];
		CCSprite* Image = [[CCSprite spriteWithFile:[[m_gameManager getStageManager] getBackGournd_image]
						  ] retain];
		[Image setPosition:cpv(s.width/2, s.height/2)];
		[self addChild:Image z:0 tag:0];	
		
	}
	return self;
}

-(void) dealloc
{
	[super dealloc];
}
@end


// ****************************************************************
@implementation GameFieldLineLayer

#define REDALART_COUNT_MAX 20

-(id) initWithGameManager: (GameManager *) gameManager
{
	self = [super init];
	if( self != nil )
	{
		m_gameManager = gameManager;
		redAlartCounter = REDALART_COUNT_MAX;
		
		drawingCeilingY = [[m_gameManager getFieldManager] getfield_y] + [[m_gameManager getFieldManager] getfield_height];		
	}
	return self;
}

-(void)draw{
	redAlartCounter--;
	if(redAlartCounter < 0) redAlartCounter = REDALART_COUNT_MAX;
	
	// Field Manager has line object in space
	int x = [[m_gameManager getFieldManager] getfield_x];
	int y = [[m_gameManager getFieldManager] getfield_y];
	int width = [[m_gameManager getFieldManager] getfield_width];
	int height = [[m_gameManager getFieldManager] getfield_height];

	// For Red Alart
	if(redAlartCounter > REDALART_COUNT_MAX / 2 &&
	   [m_gameManager getGameRedAlart])
		glColor4f(255.0, 0.0, 0.0, 1.0);
	else {
		glColor4f([[m_gameManager getStageManager] getFieldLineColor:0],
				  [[m_gameManager getStageManager] getFieldLineColor:1],
				  [[m_gameManager getStageManager] getFieldLineColor:2],
				  [[m_gameManager getStageManager] getFieldLineColor:3]);
	}
	
	if(y+height - [m_gameManager getCeilingY] > drawingCeilingY)
		drawingCeilingY++;
	else 	if(y+height - [m_gameManager getCeilingY] < drawingCeilingY)
		drawingCeilingY--;

	glLineWidth(3.0f * iPhone_iPad);
	ccDrawLine( cpv(x,		y),					cpv(x+width,	y) );
	ccDrawLine( cpv(x+width,	y),					cpv(x+width,	drawingCeilingY ) );
	ccDrawLine( cpv(x+width,	drawingCeilingY),	cpv(x,			drawingCeilingY ) );
	ccDrawLine( cpv(x,		drawingCeilingY ),	cpv(x,			y) );
}

-(void) dealloc
{
	[super dealloc];
}
@end


// ****************************************************************
@implementation GameTopTextLayer

#define TAG_SCORE_LABEL						0
#define TAG_SCORE_LABEL_SHADOW				1
#define TAG_PAUSE_LABEL						2
#define TAG_PAUSE_LABEL_SHADOW				3
#define TAG_LOOPNUM_LABEL					4
#define TAG_LOOPNUM_LABEL_SHADOW			5
#define TAG_STAGE_TUTORIAL_LABEL			6
#define TAG_STAGE_TUTORIAL_LABEL_SHADOW		7
#define TAG_ARTISTNAME_LABEL				8
#define TAG_ARTISTNAME_LABEL_SHADOW			9
#define TAG_SONGNAME_LABEL					10
#define TAG_SONGNAME_LABEL_SHADOW			11
#define TAG_BREAKALL_LABEL					12

#define TAG_HITTHEBOX_LABEL					20
#define TAG_BOX_PLUG_BOX_LABEL				21
#define TAG_FLICK_LABEL						22
#define TAG_SIZEOVER_LABEL					23
#define TAG_HIT_THE_SAME_BOX_LABEL			24
#define TAG_LOCKED_LABEL					25
#define TAG_LIMITER_RELEASE_LABEL			26
#define	TAG_FLICKBEFORELIMIT_LABEL			27
#define TAG_DO_YOU_UNDERSTAND_LABEL			28
#define TAG_DO_YOU_UNDERSTAND_LABEL_SHADOW	28

#define TAG_CLEAR_LABEL						30
#define TAG_CLEAR_LABEL_SHADOW				31

#define TAG_GAME_OVER_LABEL					32
#define TAG_GAME_OVER_LABEL_SHADOW			33

#define TAG_BREAKSIZE_LABEL					40

#define TAG_MEMORY_LABEL					50
#define TAG_ADDING_SCORE_LABEL				100
#define TAG_TOB_LABEL						200


-(id) initWithGameManager: (GameManager *) gameManager
{
	self = [super init];
	if( self != nil )
	{
		m_gameManager = gameManager;
		relationstagestopCounter = 0;
		explosionstagestopCounter = 0;
		autoReleaseTimes = 0;
		launcher_color_counter = 0;
		
		// Game Field Size
		CGSize s = [[CCDirector sharedDirector] winSize];
		display_width = s.width;
		display_height = s.height;
		display_width_center = display_width / 2;
		display_height_center = display_height / 2;
		display_height_launcher_y = display_height / 24;
		display_score_y = (display_height + 
						   ([[m_gameManager getFieldManager] getfield_y] + [[m_gameManager getFieldManager] getfield_height]))
							/ 2;
		
		StageManager* stageManager = [m_gameManager getStageManager];
		
		// SCORE LABEL
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = [CCLabel labelWithString:[NSString stringWithFormat:@"SCORE %09d", 0]
												   fontName:@"AmericanTypewriter" fontSize:20 * iPhone_iPad];
			if(i == 0){
				[label setColor:ccc3(0 ,0 ,0)];
			}else{
				[label setColor:ccc3(255 ,255 ,255)];
			}
			[label setPosition: cpv(100, -1000)];
			[self addChild:label z:10 tag:TAG_SCORE_LABEL + i];
		}
		
		// PAUSE LABEL
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = [CCLabel labelWithString:[NSString stringWithFormat:@"PAUSE"]
												   fontName:@"AmericanTypewriter" fontSize:20 * iPhone_iPad];
			if(i == 0){
				[label setColor:ccc3(0 ,0 ,0)];
			}else{
				[label setColor:ccc3(255 ,255 ,255)];
			}
			[label setPosition: cpv(100, -1000)];
			[self addChild:label z:10 tag:TAG_PAUSE_LABEL + i];
		}

		// LOOP NUM LABEL
		int color_r = [stageManager getStageStringColor:0];
		int color_g = [stageManager getStageStringColor:1];
		int color_b = [stageManager getStageStringColor:2];
		int LoopNum = [stageManager getStageLoop];
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = [CCLabel labelWithString:[NSString stringWithFormat:@"LOOP %d", LoopNum]
										fontName:@"Helvetica" fontSize:32 * iPhone_iPad];
			[label setPosition: cpv(100, -1000)];
			if(i == 0){	// Shadow
				[label setColor:ccc3([stageManager getTextShadowColor:0],
									 [stageManager getTextShadowColor:1],
									 [stageManager getTextShadowColor:2])];
			}else{
				[label setColor:ccc3(color_r * i
										 ,color_g * i
										 ,color_b * i)];
			}
			[self addChild:label z:10 tag:TAG_LOOPNUM_LABEL + i];
		}
		
		// STAGE_TUTORIAL NUM LABEL
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label;
				label = [CCLabel labelWithString:
						 [NSString stringWithFormat:@"STAGE %d-%d",
						  [m_gameManager getStageRound],
						  [m_gameManager getStageNum]]
									  fontName:@"Helvetica" fontSize:32 * iPhone_iPad];
				[label setPosition: cpv(100, -1000)];
			if(i == 0){	// Shadow
				[label setColor:ccc3([stageManager getTextShadowColor:0],
									 [stageManager getTextShadowColor:1],
									 [stageManager getTextShadowColor:2])];
			}else{
				[label setColor:ccc3(color_r * i
									 ,color_g * i
									 ,color_b * i)];
			}
			[self addChild:label z:10 tag:TAG_STAGE_TUTORIAL_LABEL + i];
		}
		
		// ARTISTNAME LABEL
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = [CCLabel labelWithString:[[NSString alloc] initWithFormat:@"%@", [[m_gameManager getStageManager] getBgm_artist_name]]
											 fontName:@"Helvetica" fontSize:24 * iPhone_iPad];
			if(i == 0){	// Shadow
				[label setColor:ccc3([stageManager getTextShadowColor:0],
									 [stageManager getTextShadowColor:1],
									 [stageManager getTextShadowColor:2])];
			}else{
				[label setColor:ccc3([stageManager getArtistNameStringColor:0] * i,
									 [stageManager getArtistNameStringColor:1] * i,
									 [stageManager getArtistNameStringColor:2] * i)];
			}
			[label setPosition: cpv(100, -1000)];
			[self addChild:label z:10 tag:TAG_ARTISTNAME_LABEL + i];
		}
		
		// SONGNAME LABEL
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = [CCLabel labelWithString:[[NSString alloc] initWithFormat:@"%@", [[m_gameManager getStageManager] getBgm_song_name]]
											 fontName:@"Helvetica" fontSize:24 * iPhone_iPad];
			if(i == 0){	// Shadow
				[label setColor:ccc3([stageManager getTextShadowColor:0],
									 [stageManager getTextShadowColor:1],
									 [stageManager getTextShadowColor:2])];
			}else{
				[label setColor:ccc3([stageManager getArtistNameStringColor:0] * i,
									 [stageManager getArtistNameStringColor:1] * i,
									 [stageManager getArtistNameStringColor:2] * i)];
			}
			[label setPosition: cpv(100, -1000)];
			[self addChild:label z:10 tag:TAG_SONGNAME_LABEL + i];
		}
		
		// ADDING SCORE LABEL
		if([stageManager IsTutorialStage] == FALSE){	// For Game Only (Tutorial is out of scope)
			for(int i = 0; i < 7; i++){
				if(iPhone_iPad == iPhone){
					CCLabelAtlas* label = [[CCLabelAtlas labelAtlasWithString:[NSString stringWithFormat:@"0"]
														  charMapFile:@"NUMBER_IMAGES_iPhone.png"
															itemWidth:23 * iPhone_iPad
														   itemHeight:40 * iPhone_iPad startCharMap:'.'] retain];
					[label setPosition: cpv(100, -1000)];
					[self addChild:label z:9 tag:TAG_ADDING_SCORE_LABEL + i];
					
				}else{	// iPad. It needs to replace with CCLabelAtlas when iPad will be upgraded to iOS4 because the performance of CCLabel on iOS4 is bad.
					for(int j = 0; j <= 1; j++){	// Shadow
						CCLabel* label = [CCLabel labelWithString:[NSString stringWithFormat:@"0"]
														 fontName:@"AmericanTypewriter"
														 fontSize:40 * iPhone_iPad];
						[label setColor:ccc3(255*j ,255*j ,255*j)];
						[label setPosition: cpv(100, -1000)];
					[self addChild:label z:9 tag:TAG_ADDING_SCORE_LABEL + i * 2 + j];
					}
				}
			}
		}

		// ADDING TOB LABEL
		for(int i = 0; i < 5; i++){
			CCLabel* label = [CCLabel labelWithString:@"TOB"
										   fontName:@"Zapfino"
										   fontSize:40 * iPhone_iPad];
			[label setColor:ccc3(0 ,0 ,0)];
			[label setPosition: cpv(100, -1000)];
			[self addChild:label z:8 tag:TAG_TOB_LABEL + i];
		}
		
		// "BREAK SIZE" LABEL
		for(int i = 0; i < 5; i++){
			CCLabel* label = [CCLabel labelWithString:
						  [NSString stringWithFormat:@"Break Size", [m_gameManager getStageNum]]
										 fontName:@"AmericanTypewriter" fontSize:28 * iPhone_iPad];
			[label setColor:ccc3(255 ,0 ,0)];
			[label setPosition: cpv(100, -1000)];
			[self addChild: label z:3 tag:TAG_BREAKSIZE_LABEL+i];
		}
		
		// "BREAK ALL" LABEL
		CCLabel* label = [CCLabel labelWithString:@"Break All"
							  fontName:@"AmericanTypewriter"
							  fontSize:28 * iPhone_iPad];
		[label setColor:ccc3(0 ,0 ,0)];
		[label setPosition: cpv(100, -1000)];
		[self addChild:label z:10 tag:TAG_BREAKALL_LABEL];
		

		
		// Clear LABEL
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = [CCLabel labelWithString:@"Clear"
											 fontName:@"Zapfino"
											 fontSize:40 * iPhone_iPad];
			if(i == 0){	// Shadow
				[label setColor:ccc3([stageManager getTextShadowColor:0],
									 [stageManager getTextShadowColor:1],
									 [stageManager getTextShadowColor:2])];
				
			}else{
				[label setColor:ccc3([stageManager getClearStringColor:0],
									 [stageManager getClearStringColor:1],
									 [stageManager getClearStringColor:2])];
			}
			[label setPosition: cpv(100, -1000)];
			[self addChild:label z:10 tag:TAG_CLEAR_LABEL + i];
		}
		
		// Game Over LABEL
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = [CCLabel labelWithString:@"Game Over"
											 fontName:@"Zapfino"
											 fontSize:40 * iPhone_iPad];
			if(i == 0){	// Shadow
				[label setColor:ccc3([stageManager getTextShadowColor:0],
									 [stageManager getTextShadowColor:1],
									 [stageManager getTextShadowColor:2])];
			}else{
				[label setColor:ccc3([stageManager getGameOverStringColor:0],
									 [stageManager getGameOverStringColor:1],
									 [stageManager getGameOverStringColor:2])];
			}
			[label setPosition: cpv(100, -1000)];
			[self addChild:label z:10 tag:TAG_GAME_OVER_LABEL + i];
		}
		
		// MEMORY LABEL
		label = [CCLabel labelWithString:[NSString stringWithFormat:@"0"]
										 fontName:@"AmericanTypewriter" fontSize:15 * iPhone_iPad];
		[label setColor:ccc3([stageManager getArtistNameStringColor:0],
							 [stageManager getArtistNameStringColor:1],
							 [stageManager getArtistNameStringColor:2])];
		[label setPosition: cpv([stageManager getBgm_artist_name_position_x], display_height / 24 ) ];
		[self addChild:label z:10 tag:TAG_MEMORY_LABEL];

		
		game_counter = 0;
	}
	return self;
}

-(void)draw{
	StageManager* stageManager = [m_gameManager getStageManager];
	ScoreManager* scoreManager = [m_gameManager getScoreManager];
	Relations* relations = [m_gameManager getRelations];
	
	if([m_gameManager getGamePause] == FALSE)
		if(	game_counter < 10000000)	game_counter++;
		
	launcher_color_counter++;
	if(	launcher_color_counter > 20) launcher_color_counter = 0;
	
	if([m_gameManager getGamePause] == FALSE){
		if([stageManager IsTutorialStage] == FALSE){
			// SCORE String
			int displaying_score = [m_gameManager getDisplayingScore];
			for(int i = 0; i <= 1; i++){
				CCLabel* label = (CCLabel*)[self getChildByTag:TAG_SCORE_LABEL + i];
				[label setString:[NSString stringWithFormat:@"SCORE %09d", displaying_score]];
				[label setPosition: cpv(display_width_center + i, display_score_y + i)];
			}

			// PAUSE OUT
			for(int i = 0; i <= 1; i++){	// Shadow
				CCLabel* label = (CCLabel*)[self getChildByTag:TAG_PAUSE_LABEL + i];
				[label setPosition: cpv(100, -1000)];
			}
		}
	}else{
		// SCORE OUT
		for(int i = 0; i <= 1; i++){
			CCLabel* label = (CCLabel*)[self getChildByTag:TAG_SCORE_LABEL + i];
			[label setPosition: cpv(100, -1000)];
		}
		// PAUSE
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = (CCLabel*)[self getChildByTag:TAG_PAUSE_LABEL + i];
			[label setPosition: cpv(display_width_center + i, display_score_y + i)];
		}
	}

	// Loop / Stage Number
	if( game_counter < 200){
		// Loop
		int LoopNum = [stageManager getStageLoop];
		if( LoopNum > 1 && [stageManager IsTutorialStage] == FALSE){
			for(int i = 0; i <= 1; i++){
				CCLabel* label = (CCLabel*)[self getChildByTag:TAG_LOOPNUM_LABEL + i];
				[label setPosition: cpv(display_width_center + i, display_height * 6 / 8 + i)];
			}
		}else{	// OUT
		}
		
		// Stage
			for(int i = 0; i <= 1; i++){	// Shadow
				CCLabel* label = (CCLabel*)[self getChildByTag:TAG_STAGE_TUTORIAL_LABEL + i];
				[label setPosition: cpv(display_width_center + i, display_height * 5 / 8 + i)];
			}
	}else{	// Set on bottom left
		for(int i = 0; i <= 1; i++){		// Shadow
			CCLabel* label = (CCLabel*)[self getChildByTag:TAG_LOOPNUM_LABEL + i];
			[label setPosition: cpv(100, -1000)];
			
			if([m_gameManager getDisplaMemory] == FALSE){
				label = (CCLabel*)[self getChildByTag:TAG_STAGE_TUTORIAL_LABEL + i];
				[label setScale:(float)0.5];
				[label setPosition: cpv(display_width / 5 + i, display_height / 24 + i) ];
			}else{	// Displaying Memory
				label = (CCLabel*)[self getChildByTag:TAG_STAGE_TUTORIAL_LABEL + i];
				[label setPosition: cpv(100, -1000)];
			}
		}		
	}
	
	// Artist Name, Song Name
		float name_size;
		float offset;
		if(game_counter < 250 && [m_gameManager getStageNum] == 1){
			offset = 0;
			name_size = 24;
		}else if((game_counter >= 250 && game_counter < 300) && [m_gameManager getStageNum] == 1){
			offset = ((float)game_counter - 250) / 50;
			name_size = 24 - 10 * offset;
		}else{
			offset = 1;
			name_size = 14;
		}
		
		// Artist Name
		int x = display_width_center - (display_width_center - [stageManager getBgm_artist_name_position_x]) * offset;
		int y = display_height_center - (display_height_center - [stageManager getBgm_artist_name_position_y]) * offset;
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = (CCLabel*)[self getChildByTag:TAG_ARTISTNAME_LABEL + i];
			[label setScale:(float) name_size / (float) 24];
			[label setPosition: cpv(x + i, y + i)];
		}
		
		// Song Name
		y = y + (name_size + 4)  * iPhone_iPad;
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = (CCLabel*)[self getChildByTag:TAG_SONGNAME_LABEL + i];
			[label setScale:(float) name_size / (float) 24];
			[label setPosition: cpv(x + i, y + i)];
		}
	
	// Adding Score
		for(int i = 0; i < 7; i++){	// limitter
			if(i < [scoreManager getNumOfAddingScore]){
				int x = [scoreManager getAddingScore_x:i];
				int y = [scoreManager getAddingScore_y:i];
				int i_score = [scoreManager getAddingScore_addingScore_displaying:i];
				float font_size = (float)40 * [scoreManager getAddingScore_addingScore_displaying_counter:i]/ADDINGSCORE_DISPLAYING_COUNTER_MAX;
				if(i_score > 0) {
					
					int numOfLetters = 1;
					if(i_score > 9)		numOfLetters = 2;
					if(i_score > 99)	numOfLetters = 3;
					if(i_score > 999)	numOfLetters = 4;
					if(i_score > 9999)	numOfLetters = 5;
					
					if(iPhone_iPad == iPhone){
						CCLabelAtlas* label = (CCLabelAtlas*)[self getChildByTag:TAG_ADDING_SCORE_LABEL + i ];
						[label setString:[NSString stringWithFormat:@"%d", i_score]];
						float font_scale = font_size / (float)40;
						[label setScale:font_scale];

						[label setPosition: cpv(x - ((numOfLetters * 23 * iPhone_iPad) * font_scale / 2),
												y - ((               40 * iPhone_iPad) * font_scale / 2))];
					}else{	// iPad
						for(int j = 0; j <= 1; j++){	// Shadow
							CCLabel* label = (CCLabel*)[self getChildByTag:TAG_ADDING_SCORE_LABEL + i * 2 + j ];
							[label setString:[NSString stringWithFormat:@"%d", i_score]];
							float font_scale = font_size / (float)40;
							[label setScale:font_scale];
							[label setPosition: cpv( x + j, y + j)];
						}
					}
				}
			}else{
				if(iPhone_iPad == iPhone){
					CCLabelAtlas* label = (CCLabelAtlas*)[self getChildByTag:TAG_ADDING_SCORE_LABEL + i ];
					[label setPosition: cpv(100, -1000)];
				}else{	// iPad
					for(int j = 0; j <= 1; j++){	// Shadow
						CCLabel* label = (CCLabel*)[self getChildByTag:TAG_ADDING_SCORE_LABEL + i * 2 + j ];
						[label setPosition: cpv(100, -1000)];
					}
				}
			}
		}
	
	// TOB
	int boxCounter, relation_XFrom, relation_YFrom, relation_XTo, relation_YTo;
	for(int i = 0; i < 5; i++){
		if(i < [relations getCount]){
			boxCounter = [relations getRelationCounter:i];
			if(boxCounter > 0){
				relation_XFrom =	[relations getboxObjectXFrom_index:i];
				relation_YFrom =	[relations getboxObjectYFrom_index:i];
				relation_XTo =		[relations getboxObjectXTo_index:i];
				relation_YTo =		[relations getboxObjectYTo_index:i];

				int tobFontSize;
				if(boxCounter > RELATION_COUNT_MAX * 9 / 10){
					tobFontSize = (float) 40 * (RELATION_COUNT_MAX - boxCounter) / (RELATION_COUNT_MAX / 10);
				}else if (boxCounter > RELATION_COUNT_MAX  / 2){
					tobFontSize = 40;
				}else{
					tobFontSize = (float) 40 * boxCounter / (RELATION_COUNT_MAX / 2);
				}
			
				CCLabel* label = (CCLabel*)[self getChildByTag:TAG_TOB_LABEL + i];
				[label setScale:tobFontSize / (float)40];
				[label setColor:ccc3(0 ,(float)boxCounter*255/RELATION_COUNT_MAX ,0)];
				[label setPosition: cpv((relation_XFrom+relation_XTo)/2, (relation_YFrom+relation_YTo)/2)];
			}
		}else{	// OUT OF FIELD
			CCLabel* label = (CCLabel*)[self getChildByTag:TAG_TOB_LABEL + i];
			[label setPosition: cpv(100, -1000)];
		}
	}
	
	// Break Size
	int maxNumOfBreakSize = 0;
	for(int i = 0; i < [m_gameManager getBoxObject_Count]; i++){
		if(maxNumOfBreakSize < 5){
			boxObject* box = [m_gameManager getBoxObjectFrom_index:i];
			int x = [box getX];
			int y = [box getY];
			int diff = BOX_SIZE_PIXEL * EXPLODE_SCALE/*%*/ / 100 / 2;

			if(([stageManager getStageRound] <= 1        && [box isREDLINE_Tutorial] == TRUE) &&
			   ([stageManager getStageNum] == 1 || [stageManager getStageNum] == 2)){			// Stage 1-1, 1-2 Guide
				CCLabel* label = (CCLabel*)[self getChildByTag:TAG_BREAKSIZE_LABEL + maxNumOfBreakSize];
				[label setPosition: cpv(x, y + diff + 28 / 2 * iPhone_iPad)];
				maxNumOfBreakSize++;
			}
		}
	}
	for(int i = maxNumOfBreakSize; i < 5; i++){		// OUT OF FIELD
		CCLabel* label = (CCLabel*)[self getChildByTag:TAG_BREAKSIZE_LABEL + i];
		[label setPosition: cpv(100, -1000)];
	}
	
	// Break all
	if(game_counter >= 200 && [m_gameManager getwasFlicked] == TRUE &&
	   [stageManager getStageRound] <= 1 && [m_gameManager getGameClear] == FALSE &&
		([stageManager getStageNum] == 1 || [stageManager getStageNum] == 2)){
		CCLabel* label = (CCLabel*)[self getChildByTag:TAG_BREAKALL_LABEL];
		[label setPosition: cpv(display_width / 2, display_height * 0.3)];
	}else{		// OUT OF FIELD
		CCLabel* label = (CCLabel*)[self getChildByTag:TAG_BREAKALL_LABEL];
		[label setPosition: cpv(100, -1000)];
	}
	


	// Game Clear
	if([m_gameManager getGameClear] == TRUE && [scoreManager getNumOfAddingScore] == 0){
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = (CCLabel*)[self getChildByTag:TAG_CLEAR_LABEL + i];
			[label setPosition: cpv(display_width_center +i, display_height_center+i)];
		}
	}
	
	// Game Lost
	if([m_gameManager getGameLost] == TRUE){
		for(int i = 0; i <= 1; i++){	// Shadow
			CCLabel* label = (CCLabel*)[self getChildByTag:TAG_GAME_OVER_LABEL + i];
			[label setPosition: cpv(display_width_center +i, display_height_center+i)];
		}
	}

	// MEMORY	
	if([m_gameManager getDisplaMemory] == TRUE){
		struct vm_statistics a_vm_info;
		mach_msg_type_number_t a_count = HOST_VM_INFO_COUNT;
		host_statistics( mach_host_self(), HOST_VM_INFO, (host_info_t)&a_vm_info ,&a_count);
		
		CCLabel* label = (CCLabel*)[self getChildByTag:TAG_MEMORY_LABEL];
		[label setString:[NSString stringWithFormat:@"%06f MB", (float)a_vm_info.free_count * vm_page_size / 1000000]];
		[label setPosition: cpv(display_width / 5, display_height / 24 ) ];
	}else{
		CCLabel* label = (CCLabel*)[self getChildByTag:TAG_MEMORY_LABEL];
		[label setPosition: cpv(100, -1000) ];
	}
}

-(void) dealloc
{
	[super dealloc];
}
@end


// ****************************************************************
@implementation GameLineAndStringLayer

-(id) initWithGameManager: (GameManager *) gameManager{
	self = [super init];
	if( self != nil )
	{
		m_gameManager = gameManager;
		m_Relations = [m_gameManager getRelations];
		m_ScoreManager = [m_gameManager getScoreManager];
		m_StageManager = [m_gameManager getStageManager];
		
		redAlartCounter = REDALART_COUNT_MAX;
		tentative_x = 10;
		launcher_color_counter = 0;
		arrow_move_counter = 0;
		game_counter = 0;
		
		// Field Size
		CGSize s = [[CCDirector sharedDirector] winSize];
		display_width = s.width;
		display_height = s.height;
		display_width_center = display_width / 2;
		display_height_center = display_height / 2;
		display_height_launcher_y = display_height / 24;
		display_score_y = (display_height + 
						   ([[m_gameManager getFieldManager] getfield_y] + [[m_gameManager getFieldManager] getfield_height]))
						/ 2;

		m_text_shadow_color_r = [m_StageManager getTextShadowColor:0];
		m_text_shadow_color_g = [m_StageManager getTextShadowColor:1];
		m_text_shadow_color_b = [m_StageManager getTextShadowColor:2];
	}
	return self;
}



-(void)draw{
	redAlartCounter--;
	if(redAlartCounter < 0) redAlartCounter = REDALART_COUNT_MAX;

	launcher_color_counter++;
	if(	launcher_color_counter > 20) launcher_color_counter = 0;

	if([m_gameManager getGamePause] == FALSE)
		if(	game_counter < 10000000)	game_counter++;

	// Clean up
	[self removeAllChildrenWithCleanup:NO];

	// draw Relations
	int boxCounter, relation_XFrom, relation_YFrom, relation_XTo, relation_YTo;
	for(int i = 0; i < [m_Relations getCount]; i++){
		boxCounter = [m_Relations getRelationCounter:i];
		if(boxCounter > 0){
			relation_XFrom =	[m_Relations getboxObjectXFrom_index:i];
			relation_YFrom =	[m_Relations getboxObjectYFrom_index:i];
			relation_XTo =		[m_Relations getboxObjectXTo_index:i];
			relation_YTo =		[m_Relations getboxObjectYTo_index:i];
			// Line
			glColor4f(0.0,	//R
					  (float)boxCounter*255/RELATION_COUNT_MAX,	//G
					  0.0, //B
					  (float)boxCounter/RELATION_COUNT_MAX);    // Alpha
			glLineWidth(3.0f * (float)boxCounter/RELATION_COUNT_MAX * iPhone_iPad);
			ccDrawLine( cpv(relation_XFrom, relation_YFrom),
					 cpv(relation_XTo, relation_YTo) );

		}
	}

	// Draw box expand warning red line
	glColor4f((float)100, 0.0, 0.0, (float)0.5);    // Alpha
	glLineWidth(4.0f * iPhone_iPad);
	for(int i = 0; i < [m_gameManager getBoxObject_Count]; i++){
		boxObject* box = [m_gameManager getBoxObjectFrom_index:i];
		int x = [box getX];
		int y = [box getY];
		int diff = BOX_SIZE_PIXEL * EXPLODE_SCALE/*%*/ / 100 / 2;
		if([box isREDLINE] == TRUE || 
			(([m_StageManager getStageRound] <= 1  && [box isREDLINE_Tutorial] == TRUE) &&
			([m_StageManager getStageNum] == 1 || [m_StageManager getStageNum] == 2))){	// Stage 1-1, 1-2
			ccDrawLine( cpv(x - diff, y - diff), cpv(x + diff, y - diff));
			ccDrawLine( cpv(x + diff, y - diff), cpv(x + diff, y + diff));
			ccDrawLine( cpv(x - diff, y + diff), cpv(x + diff, y + diff));
			ccDrawLine( cpv(x - diff, y - diff), cpv(x - diff, y + diff));		
		}
	}
	
	// Launching Box Line
	float diff = BOX_SIZE_PIXEL / 2 * AUTO_RELEASE_BOX_SIDE / 100;
		glColor4f((float)0.0, 255.0 , 0.0, (float)1.0 * launcher_color_counter / 20);    // Alpha
		glLineWidth(1.5f * iPhone_iPad);

		ccDrawLine( cpv(display_width_center -	diff,	display_height_launcher_y -	diff + 25 * iPhone_iPad),
			 cpv(	display_width_center +		diff,	display_height_launcher_y -	diff + 25 * iPhone_iPad));
		ccDrawLine( cpv(display_width_center +	diff,	display_height_launcher_y -	diff + 25 * iPhone_iPad),
			 cpv(	display_width_center +		diff,	display_height_launcher_y +	diff + 25 * iPhone_iPad));
		ccDrawLine( cpv(display_width_center -	diff,	display_height_launcher_y +	diff + 25 * iPhone_iPad),
			 cpv(	display_width_center +		diff,	display_height_launcher_y +	diff + 25 * iPhone_iPad));
		ccDrawLine( cpv(display_width_center -	diff,	display_height_launcher_y -	diff + 25 * iPhone_iPad),
			 cpv(	display_width_center -		diff,	display_height_launcher_y +	diff + 25 * iPhone_iPad));		


	   
	   
	// draw Flick Line
	double LineRate;
	if([m_gameManager getFlickLine_count] > max_flick_count * 9 / 10){
		LineRate = ((float)max_flick_count - (float)[m_gameManager getFlickLine_count]) / (float)(max_flick_count / 10);
	}else if ([m_gameManager getFlickLine_count] > max_flick_count  / 2){
		LineRate = 1;
	}else{
		LineRate = (float)[m_gameManager getFlickLine_count] / (float)(RELATION_COUNT_MAX / 2);
	}
	if(iPhone_iPad == iPad) LineRate = LineRate / 2;	// For iPad
	glColor4f((float)255.0, 0.0 , 0.0, (float)1.0 * launcher_color_counter / 20);    // Alpha
	glLineWidth(2.5f * iPhone_iPad);
	ccDrawLine( cpv(display_width_center,	display_height_launcher_y + 25 * iPhone_iPad),
			 cpv(display_width_center + [m_gameManager getFlickLine_vx] * LineRate * iPhone_iPad,
				 display_height_launcher_y + 25 * iPhone_iPad - [m_gameManager getFlickLine_vy] * LineRate * iPhone_iPad));
}

-(void) dealloc
{
	[super dealloc];
}
@end


@implementation GameLayer

@synthesize m_touch_x, m_touch_y, m_touch_time;

-(id) initWithGameManager: (GameManager *) gameManager{
	self = [super init];
	if(self != nil){
		isTouchEnabled = YES;
		m_gameManager = gameManager;
		isFlicked = FALSE;
		
		[m_gameManager resetStageforNext:self];		// Init Field

		//Update Chipmunk
		cpSpaceAddCollisionPairFunc([m_gameManager getcpSpace], 0, 0, &boxCollision, self);

		[self schedule: @selector(step:)];

		countstart_time = 0;
		countgameover_time = 0;

		[self addChild: [[GameBackGroundLayer node]			initWithGameManager:m_gameManager] z:0];
		[self addChild: [[GameFieldLineLayer node]			initWithGameManager:m_gameManager] z:1];
		[self addChild: [[GameLineAndStringLayer node]		initWithGameManager:m_gameManager] z:1000000];
		[self addChild: [[GameTopTextLayer node]			initWithGameManager:m_gameManager] z:1000001];
		
		// Touch Detect
		self.m_touch_x		=	[[NSMutableArray alloc] init];;			// touch log
		self.m_touch_y		=	[[NSMutableArray alloc] init];;			// touch log
		self.m_touch_time	=	[[NSMutableArray alloc] init];;			// touch log (msec)
		
		// Menu Back Button
		CCMenuItem* backToMenuButtonItem = [CCMenuItemImage itemFromNormalImage:@"Button_Menu.png"
											   selectedImage:@"Button_Menu_Selected.png" 
													  target:self selector:@selector(backToMenuButton:)];
		[backToMenuButtonItem	setScale:0.5 * iPhone_iPad];
		
		CCMenu * menu = [CCMenu menuWithItems:backToMenuButtonItem, nil];
		[menu alignItemsVertically];
		[self addChild:menu z:10000000 tag: 0];
		
		CGSize s = [[CCDirector sharedDirector] winSize];
		[menu setPosition:cpv(s.width / 2, s.height * 2)];	// Out Of display
		
		display_score_y = (s.height + 
						   ([[m_gameManager getFieldManager] getfield_y] + [[m_gameManager getFieldManager] getfield_height])) / 2;
				
		CCSprite* Image = [CCSprite spriteWithFile:@"Button_Pause.png"];
		Image.scale = 0.4 * iPhone_iPad;
		[Image setPosition:cpv(15 * iPhone_iPad, display_score_y)];	// iPhone 15(30x30), iPad 30(60x60)
		[self addChild:Image z:0 tag:1];
		
		Image = [CCSprite spriteWithFile:@"Button_Resume.png"];
		Image.scale = 0.4 * iPhone_iPad;
		[Image setPosition:cpv(s.width / 2, s.height * 2)];			// iPhone 15(30x30), iPad 30(60x60)		Out Of display
		[self addChild:Image z:0 tag:2];
		
		
		game_start_time_test = [NSDate timeIntervalSinceReferenceDate];	// For Test
		
		display_width = s.width;
	}

	return self;
}

-(void) backToMenuButton: (id)sender{
	[self backToMenu];
}

-(void) backToMenu{
	[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:[NSString stringWithFormat:@"Game_BGM%d",[m_gameManager getStageRound]]];
	[[MyOpenAL sharedMyOpenAL] releaseSoundWithKey:[NSString stringWithFormat:@"Game_BGM%d",[m_gameManager getStageRound]]];
	[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:@"GameOver"];
	[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:@"GameOverWarning"];
	
	CCScene *s = [CCScene node];
	[s addChild: [[MenuLayer node] initWithGameManager:m_gameManager]];
	[[CCDirector sharedDirector] replaceScene: s];
}

-(void) step: (ccTime) delta
{
	if([m_gameManager getGamePause] == TRUE) return;
	
	if([m_gameManager getNumOfObjectsOnField] == 0) [m_gameManager setGameClear];

	if(isFlicked && [m_gameManager getGameClear] == FALSE && [m_gameManager getGameLost] == FALSE){
		if( [m_gameManager launchBox_vx:(float)flick_vx vy:(float)flick_vy] == TRUE)	// if flicked
			[m_gameManager setFlickLine_vx:flick_vx/4 vy:-flick_vy/4];
		isFlicked = FALSE;
	}

	int steps = 2;							// Normal Speed

		cpFloat dt = delta/(cpFloat)steps;
		cpSpace* space = [m_gameManager getcpSpace];
	
		for(int i=0; i<steps; i++){
			if([m_gameManager getGameClear] == FALSE && [m_gameManager getGameLost] == FALSE){
				[m_gameManager stepFieldAnimation:self stageHalt:FALSE];
			}else{
				[m_gameManager stepFieldAnimation:self stageHalt:TRUE];
			}
			cpSpaceStep(space, dt);
		}
	
		cpSpaceHashEach(space->activeShapes, &eachShape, nil);
		cpSpaceHashEach(space->staticShapes, &eachShape, nil);
	
	// For Test
	if(NO_PLAY_MODE == 1){
		[m_gameManager setGameClear];
	}
	
	// FOR PLAY TEST
//	if ([NSDate timeIntervalSinceReferenceDate] - game_start_time_test > 30.0){
//		[m_gameManager setGameClear];
//	}
	
	// Detect Game Over
	if([m_gameManager getGameLost] == TRUE){
		if(countgameover_time == 0)
			countgameover_time = [NSDate timeIntervalSinceReferenceDate];
	}
		   
	if([m_gameManager getGameClear] == TRUE && [[m_gameManager getScoreManager] getNumOfAddingScore] == 0){
		if(countstart_time == 0)
			countstart_time = [NSDate timeIntervalSinceReferenceDate];
		
		// Sleep
		if ([NSDate timeIntervalSinceReferenceDate] - countstart_time > 4.0){	// 3.0 sec sleep
			countstart_time = 0;
			[self unschedule: @selector(step:)];
			
			CCScene *s = [CCScene node];
			int stageRound = [m_gameManager getStageRound];
			if( [m_gameManager incrementStage] == TRUE){			// If stage round is incremented
				[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:[NSString stringWithFormat:@"Game_BGM%d",stageRound]];
				[[MyOpenAL sharedMyOpenAL] releaseSoundWithKey:[NSString stringWithFormat:@"Game_BGM%d",stageRound]];
				[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:@"GameOver"];
				[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:@"GameOverWarning"];
				
				[s addChild: [[TitleLayer node] initWithGameManager:m_gameManager]];
				[[CCDirector sharedDirector] replaceScene: s];
				return;
			}else{													// If stage round is not incremented
				[s addChild: [[GameLayer node] initWithGameManager:m_gameManager]];
				[[CCDirector sharedDirector] replaceScene: s];
				return;
			}
		}
	}
	
}

-(void) collision_detect:(cpShape *) shapeA withShape:(cpShape *) shapeB{
	[m_gameManager setRelations:shapeA withShape:shapeB];
}


-(void) ccTouchesBegan:(NSSet*)touches withEvent:(UIEvent *)event
{
	UITouch *touch = [touches anyObject];
	CGPoint touch_point = [touch locationInView: [touch view]];
	touch_point = [[CCDirector sharedDirector] convertToGL:touch_point];
	
	NSTimeInterval start_time = [NSDate timeIntervalSinceReferenceDate];
	[self.m_touch_x addObject:[NSNumber numberWithDouble:touch_point.x]];
	[self.m_touch_y addObject:[NSNumber numberWithDouble:touch_point.y]];
	[self.m_touch_time addObject:[NSNumber numberWithDouble:start_time]];

	start_time_test = start_time;

}

-(void) ccTouchesMoved:(NSSet*)touches withEvent:(UIEvent *)event
{
	UITouch *touch = [touches anyObject];
	CGPoint touch_point = [touch locationInView: [touch view]];
	touch_point = [[CCDirector sharedDirector] convertToGL:touch_point];
	
	NSTimeInterval moving_time = [NSDate timeIntervalSinceReferenceDate];
	[self.m_touch_x addObject:[NSNumber numberWithDouble:touch_point.x]];
	[self.m_touch_y addObject:[NSNumber numberWithDouble:touch_point.y]];
	[self.m_touch_time addObject:[NSNumber numberWithDouble:moving_time]];
}


-(void) ccTouchesEnded:(NSSet*)touches withEvent:(UIEvent*)event
{
	if([m_gameManager getGameLost] == TRUE &&		// Game Over
	   [NSDate timeIntervalSinceReferenceDate] - countgameover_time > 4.0){	// 4.0 sec wait
		[self backToMenu];
		return;
	}
	
	NSTimeInterval end_time = [NSDate timeIntervalSinceReferenceDate];

    UITouch *touch = [touches anyObject];
	CGPoint touch_point = [touch locationInView: [touch view]];
	touch_point = [[CCDirector sharedDirector] convertToGL:touch_point];
	
	isFlicked = false;
	
	// DETECT PAUSE TOUCH , NO MOVE, and same point touch and release
	CGSize s = [[CCDirector sharedDirector] winSize];
	if([self.m_touch_time count] == 1){
		int height_line_y = [[m_gameManager getFieldManager] getfield_y] +
		[[m_gameManager getFieldManager] getfield_height];
		if( touch_point.y									> height_line_y &&
		    [[self.m_touch_y objectAtIndex:0] doubleValue]	> height_line_y ){
			// PAUSE
			if([m_gameManager getGamePause] == TRUE){
				[m_gameManager setGamePause:FALSE];								// Resume
				CCNode *menu_sprite = [self getChildByTag:0];					// Menu Button
				[menu_sprite setPosition:cpv(s.width / 2, s.height * 2)];
				menu_sprite = [self getChildByTag:1];							// Pause Button		(Display)
				[menu_sprite setPosition:cpv(15 * iPhone_iPad, display_score_y)];
				menu_sprite = [self getChildByTag:2];
				[menu_sprite setPosition:cpv(s.width / 2, s.height * 2)];		// Resume Button
			}else{
				[m_gameManager setGamePause:TRUE];								// Pause
				CCNode *menu_sprite = [self getChildByTag:0];					// Menu Button	(Display)
				[menu_sprite setPosition:cpv(s.width / 2, s.height / 2)];
				menu_sprite = [self getChildByTag:1];							// Pause Button
				[menu_sprite setPosition:cpv(s.width / 2, s.height * 2)];
				menu_sprite = [self getChildByTag:2];							// Resume Button	(Display)
				[menu_sprite setPosition:cpv(15 * iPhone_iPad, display_score_y)];
			}
		}
		
	}
	
	// Touch Log to detect flick
	if([m_gameManager getGamePause] == FALSE){
		for(int i = 0; i < [self.m_touch_time count]; i++){
			double diffX = touch_point.x - [[self.m_touch_x objectAtIndex:i] doubleValue];
			double diffY = touch_point.y - [[self.m_touch_y objectAtIndex:i] doubleValue];
			double dist = sqrt(diffX * diffX + diffY * diffY);
			double diff_time = end_time - [[self.m_touch_time objectAtIndex:i] doubleValue];
			if(diff_time < 0.1 && dist > 20){
				isFlicked = true;
				[m_gameManager setwasFlicked:true];
				if(diff_time < 0.05) diff_time = 0.05;
				double coefficient = (double)0.1 / diff_time;
				coefficient = coefficient * iPhone_iPad;
				flick_vx = diffX * 4 * coefficient;
				flick_vy = diffY * 4 * coefficient;
				if(flick_vx * flick_vx + flick_vy * flick_vy > 1500 * 1500 * iPhone_iPad * iPhone_iPad){
//					if(abs(flick_vx)<1.0){										// Remove comment out to execute on emulator to avoid exception of EXC_ARITHMETIC signal, not tested code on device
//						if(flick_vy > 0) flick_vy =  1500 * iPhone_iPad;
//						if(flick_vy < 0) flick_vy = -1500 * iPhone_iPad;
//					}else if(abs(flick_vy)<1.0){
//						if(flick_vx > 0) flick_vx =  1500 * iPhone_iPad;
//						if(flick_vx < 0) flick_vx = -1500 * iPhone_iPad;
//					}else{
					float atangent = atan(abs(flick_vy)/abs(flick_vx));					// ***** Sometimes App stops by exception of EXC_ARITHMETIC signal *****
					if(flick_vx > 0) flick_vx =  1500 * iPhone_iPad * cos(atangent);
					if(flick_vx < 0) flick_vx = -1500 * iPhone_iPad * cos(atangent);
					if(flick_vy > 0) flick_vy =  1500 * iPhone_iPad * sin(atangent);
					if(flick_vy < 0) flick_vy = -1500 * iPhone_iPad * sin(atangent);
//					}															// Remove comment out to execute on emulator to avoid exception of EXC_ARITHMETIC signal, not tested code on device
				}
				break;
			}
		}
	}
	
	//MEMORY DRAW SWITCH ver 1.1
	if( touch_point.y									< 40 * iPhone_iPad &&		// Start Point is left bottom
		touch_point.x									< 40 * iPhone_iPad){
		if([[self.m_touch_x objectAtIndex:0] doubleValue]	< 40 * iPhone_iPad &&	// End Point is left bottom
		   [[self.m_touch_y objectAtIndex:0] doubleValue]	< 40 * iPhone_iPad){
			
			BOOL InTrace = TRUE;													// touch route is inside of the trace
			double max_x = 0;
			for(int i = 0; i < [self.m_touch_time count]; i++){
				if(max_x < [[self.m_touch_x objectAtIndex:i] doubleValue])			// Max x is center point
					max_x = [[self.m_touch_x objectAtIndex:i] doubleValue];
			}
			
			if(InTrace == TRUE &&
			   max_x > display_width / 2 - 20 * iPhone_iPad &&
			   max_x < display_width / 2 + 20 * iPhone_iPad){						// Detect Swtich
				if([m_gameManager getDisplaMemory] == FALSE){
					[m_gameManager setDisplaMemory:TRUE];
				}else{
					[m_gameManager setDisplaMemory:FALSE];
				}
			}
			
		}
	}

	
	
	// INIT Array
	for(int i = [self.m_touch_time count] - 1; i >= 0 ; i--){
		[self.m_touch_x removeObjectAtIndex:i];
		[self.m_touch_y removeObjectAtIndex:i];
		[self.m_touch_time removeObjectAtIndex:i];
	}
		
	return;
}

-(void) dealloc
{
	// INIT Array
	for(int i = [self.m_touch_time count] - 1; i >= 0 ; i--){
		[self.m_touch_x removeObjectAtIndex:i];
		[self.m_touch_y removeObjectAtIndex:i];
		[self.m_touch_time removeObjectAtIndex:i];
	}
	self.m_touch_x = nil;
	self.m_touch_y = nil;
	self.m_touch_time = nil;

	[[CCTextureCache sharedTextureCache] removeUnusedTextures];
	[super dealloc];
}

@end
