/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "MenuScene.h"
#import "CreditScene.h"
#import "GameScene.h"
#import "MyOpenAL.h"


#include <mach/host_info.h>
#include <mach/mach_init.h>
#include <mach/mach_host.h>

@implementation MenuScene

-(id) initWithGameManager: (GameManager *) gameManager
{
	self = [super init];
	if( self != nil )
	{
		MenuLayer* ml = [[MenuLayer node] initWithGameManager:gameManager];

		[self addChild:ml z:2];
	}

	return self;
}

-(void) dealloc
{
	[[CCTextureCache sharedTextureCache] removeUnusedTextures];
	[super dealloc];
}

@synthesize m_gameManager;
@end

@implementation MenuLayer

-(id) initWithGameManager: (GameManager *) gameManager
{
	self = [super init];
	if(self != nil)
	{
		[gameManager releaseFieldManager];

		m_gameManager =	gameManager;
		CGSize s = [[CCDirector sharedDirector] winSize];

		CCSprite* bg;
		
		if(iPhone_iPad == iPhone){
			bg = [CCSprite spriteWithFile:@"menu_background_iPhone.png"];			// iPhone
		}else{
			bg = [CCSprite spriteWithFile:@"menu_background.png"];					// iPad
		}
		bg.scale = (float) 1.0;
		[bg setPosition:cpv(s.width/2, s.height/2)];
		[self addChild:bg z:0];
		
		if(iPhone_iPad == iPhone){
			bg = [CCSprite spriteWithFile:@"menu_iPhone.png"];						// iPhone
		}else{
			bg = [CCSprite spriteWithFile:@"menu.png"];								// iPad
		}
		bg.rotation = 25;
		bg.scale = 3.0f;
		[bg setPosition:cpv(s.width/2, s.height/2)];
		[self addChild:bg z:2 tag:0];
		
		if(iPhone_iPad == iPhone){
			bg = [CCSprite spriteWithFile:@"menu_transparent_iPhone.png"];			// iPhone
		}else{
			bg = [CCSprite spriteWithFile:@"menu_transparent.png"];					// iPad
		}
		bg.rotation = 25;
		bg.scale = 0.8f;
		[bg setPosition:cpv(s.width/2, s.height/2)];
		[self addChild:bg z:1 tag:1];
		
		if(iPhone_iPad == iPhone){
			bg = [CCSprite spriteWithFile:@"logo_iPhone.png"];						// iPhone
		}else{
			bg = [CCSprite spriteWithFile:@"logo.png"];								// iPad
		}
		bg.scale = 1.0f;
		[bg setPosition:cpv(s.width/2, s.height*3/4)];
		[self addChild:bg z:2 tag:2];
		
		
		CCMenuItem* start = [CCMenuItemImage itemFromNormalImage:@"Button_Play.png"
											   selectedImage:@"Button_Play_Selected.png" 
													target:self selector:@selector(startGame:)];

		CCMenuItem* continueGame = [CCMenuItemImage itemFromNormalImage:@"Button_Continue.png"
											   selectedImage:@"Button_Continue_Selected.png" 
													  target:self selector:@selector(continueGame:)];
		
		
		CCMenuItem* credit = [CCMenuItemImage itemFromNormalImage:@"Button_Credit.png"
											   selectedImage:@"Button_Credit_Selected.png" 
													  target:self selector:@selector(credit:)];

		
		CCMenu * menu;
		if([m_gameManager getLogStageRound] != 1 || [m_gameManager getLogStageNum] != 1
			 || [[m_gameManager getStageManager] getLogStageLoop] != 1){
			menu = [CCMenu menuWithItems:start , continueGame, credit, nil];			// Continue ON
		}else {
			menu = [CCMenu menuWithItems:start , credit, nil];							// Continue OFF
		}
		
		[menu alignItemsVertically];
		[self addChild:menu z:3];
		
		int buttonY = 30 * iPhone_iPad;
		[start setPosition:cpv(0,buttonY)];			buttonY = buttonY - 60 * iPhone_iPad;
		[continueGame setPosition:cpv(0,buttonY)];	buttonY = buttonY - 60 * iPhone_iPad;
		[credit setPosition:cpv(0,buttonY)];		buttonY = buttonY - 60 * iPhone_iPad;

		[start			setScale:0.5 * iPhone_iPad];
		[continueGame	setScale:0.5 * iPhone_iPad];
		[credit			setScale:0.5 * iPhone_iPad];
		
		frame = 0;
		frame_back = 0;
		[self schedule: @selector(AnimationOfMenu:)];
		
		// Play BGM
		if([[MyOpenAL sharedMyOpenAL] loadedSoundWithKey:@"Menu_BGM"] == false){
			[self performSelectorInBackground:@selector(loadAudio) withObject:self];
		}else{
			// playing?
			if([[MyOpenAL sharedMyOpenAL] isPlayingSoundWithKey:[NSString stringWithFormat:@"Menu_BGM"]] == TRUE){
			}else{
				[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"Menu_BGM"];
			}
		}
		
		m_buttonSelected = FALSE;

		// MEMORY LABEL  ver 1.1
		CCLabel* label = [CCLabel labelWithString:[NSString stringWithFormat:@"0"]
								fontName:@"AmericanTypewriter" fontSize:15 * iPhone_iPad];
		[label setColor:ccc3([[m_gameManager getStageManager] getArtistNameStringColor:0],
							 [[m_gameManager getStageManager] getArtistNameStringColor:1],
							 [[m_gameManager getStageManager] getArtistNameStringColor:2])];
		[label setPosition: cpv(100, -1000) ];
		[self addChild:label z:10 tag:3];
	}
	return self;
}

// ver 1.1
-(void)draw{
	// MEMORY	
	if([m_gameManager getDisplaMemory] == TRUE){
		struct vm_statistics a_vm_info;
		mach_msg_type_number_t a_count = HOST_VM_INFO_COUNT;
		host_statistics( mach_host_self(), HOST_VM_INFO, (host_info_t)&a_vm_info ,&a_count);

		CCLabel* label = (CCLabel*)[self getChildByTag:3];
		[label setString:[NSString stringWithFormat:@"%06f MB", (float)a_vm_info.free_count * vm_page_size / 1000000]];
		CGSize s = [[CCDirector sharedDirector] winSize];
		[label setPosition: cpv(s.width / 5, s.height / 24 ) ];
	}else{
		CCLabel* label = (CCLabel*)[self getChildByTag:3];
		[label setPosition: cpv(100, -1000) ];
	}
}

-(void) AnimationOfMenu: (ccTime) dt
{
	CCNode *sprite = [self getChildByTag:0];
	
	frame++;
	if(frame > 360) frame = 0;
	sprite.rotation = (float) frame * 3;
	sprite.scale = 3.0f * ((360.0f-frame) / 360.0f);

	
	CCNode *sprite2 = [self getChildByTag:1];
	
	frame_back++;
	if(frame_back > 360) frame_back = 0;
	sprite2.rotation = (float) -frame_back;

	
	CCNode *sprite3 = [self getChildByTag:2];
	sprite3.scale = (1.0f + 0.8f * (float)abs(frame-180)/(float)180);
}

-(void) startGame: (id)sender
{
	if(m_buttonSelected == FALSE){
		[self performSelectorInBackground:@selector(stopAudio) withObject:self];	// async BGM stop

		[m_gameManager initStageRoundNumber];
		GameScene* gs = [[GameScene node] initWithGameManager:m_gameManager];
		[[CCDirector sharedDirector] replaceScene:gs];
		
		m_buttonSelected = TRUE;
	}
}

-(void) continueGame: (id)sender
{
	if(m_buttonSelected == FALSE){
		[self performSelectorInBackground:@selector(stopAudio) withObject:self];	// async BGM stop
	
		[m_gameManager continueStage];
		GameScene* gs = [[GameScene node] initWithGameManager:m_gameManager];
		[[CCDirector sharedDirector] replaceScene:gs];
		
		m_buttonSelected = TRUE;
	}
}


-(void) credit: (id)sender
{
	if(m_buttonSelected == FALSE){
		CreditScene* gs = [[CreditScene node] initWithGameManager:m_gameManager] ;
		[[CCDirector sharedDirector] replaceScene:gs];
		
		m_buttonSelected = TRUE;
	}
}


-(void)loadAudio{
	NSAutoreleasePool* pool;
	pool=[[NSAutoreleasePool alloc]init];
	
	if(iPhone_iPad == iPhone){
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Menu_BGM" File:@"BGM1_iPhone" Ext:@"m4a" Loop:true];
	}else{
		[[MyOpenAL sharedMyOpenAL] loadSoundWithKey:@"Menu_BGM" File:@"BGM1"		Ext:@"m4a" Loop:true];
	}
	[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"Menu_BGM"];	
	
	[pool release];
}

-(void)stopAudio{
	NSAutoreleasePool* pool;
	pool=[[NSAutoreleasePool alloc]init];
	
	while([[MyOpenAL sharedMyOpenAL] loadedSoundWithKey:@"Menu_BGM"] == false &&
		  [[MyOpenAL sharedMyOpenAL] isPlayingSoundWithKey:@"Menu_BGM"] == false ){};
	[[MyOpenAL sharedMyOpenAL] stopSoundWithKey:@"Menu_BGM"];
	[[MyOpenAL sharedMyOpenAL] releaseSoundWithKey:@"Menu_BGM"];
	
	[pool release];
}

-(void) dealloc
{
	[super dealloc];
}

@synthesize m_gameManager;
@end
