/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "Relations.h"


@implementation Relations

@synthesize boxFrom, boxTo, counter;

-(id) init {
	self = [super init];
	if(self != nil){
		self.boxFrom	=	[[NSMutableArray alloc] init];	// Relation Initialize
		self.boxTo	=		[[NSMutableArray alloc] init];	// Relation Initialize
		self.counter	=	[[NSMutableArray alloc] init];	// Relation Initialize
	}
	return self;
}

// TRUE: REGISTERED, FALSE: NOT REGISTERED
-(BOOL) addBoxFrom:(boxObject *) boxObjectfrom To:(boxObject *) boxObjectto{
	
	if( [self.counter count] > 5) return FALSE;	// limitter

	// Ignore if it has already been registered.
	for(int i=0; i < [self.counter count];i++){
		if ([self.boxFrom objectAtIndex:i] == boxObjectfrom && [self.boxTo objectAtIndex:i] == boxObjectto) return FALSE;
		if ([self.boxFrom objectAtIndex:i] == boxObjectto && [self.boxTo objectAtIndex:i] == boxObjectfrom) return FALSE;
	}
	
	// ignore if detect 0 size 
	if([boxObjectto getScale] == 0) return FALSE;
	if([boxObjectfrom getScale] == 0) return FALSE;
	
	// Find new relation
	if(boxObjectfrom && boxObjectto){
		[[MyOpenAL sharedMyOpenAL] playSoundWithKey:@"TOB"];

		[self.boxFrom	addObject:boxObjectfrom];
		[self.boxTo		addObject:boxObjectto];
		[self.counter	addObject:[NSNumber numberWithInteger:RELATION_COUNT_MAX]];	//  Set Default Value
		return TRUE;
	}
	
	return FALSE;
}

-(void) removeBoxRelation:(int) index{
	[self.boxFrom	removeObjectAtIndex:index];
	[self.boxTo		removeObjectAtIndex:index];
	[self.counter	removeObjectAtIndex:index];
}

-(void) replaceObject:(boxObject*) box with:(boxObject*) replacedBox{
	for(int i=0; i < [self.counter count];i++){
		if ([self.boxFrom	objectAtIndex:i] == box)
			[self.boxFrom	replaceObjectAtIndex:i withObject:replacedBox];
		if ([self.boxTo		objectAtIndex:i] == box)
			[self.boxTo		replaceObjectAtIndex:i withObject:replacedBox];
	}
}

-(void) stepAnimation{
	int _counter;
	
	for(int i=[self.counter count]-1; i >= 0 ;i--){
		_counter = [[self.counter objectAtIndex:i] intValue];
		_counter--;
		[self.counter replaceObjectAtIndex:i withObject:[NSNumber numberWithInteger:_counter]];
		if(_counter <= 0)
			[self removeBoxRelation:i];
	}
}

-(int) getboxObjectXFrom_index:(int) index{
	return [[self.boxFrom objectAtIndex:index] getX];
}
-(int) getboxObjectYFrom_index:(int) index{
	return [[self.boxFrom objectAtIndex:index] getY];
}

-(int) getboxObjectXTo_index:(int) index{
	return [[self.boxTo objectAtIndex:index] getX];
}
-(int) getboxObjectYTo_index:(int) index{
	return [[self.boxTo objectAtIndex:index] getY];
}

-(int) getboxObjectScaleTo_index:(int) index{
	return [[self.boxTo objectAtIndex:index] getScale];
}
-(int) getboxObjectScaleFrom_index:(int) index{
	return [[self.boxFrom objectAtIndex:index] getScale];
}


-(int) getRelationCounter:(int) index{
	return [[self.counter objectAtIndex:index] intValue];
}


-(int) getCount{
	return [self.counter count];
}

-(BOOL) hasRelation:(boxObject *) box{
	for(int i=0; i < [self.counter count];i++){
		if ([self.boxFrom objectAtIndex:i] == box || [self.boxTo objectAtIndex:i] == box) return TRUE;
	}
	return FALSE;	
}

-(void) dealloc
{
	[super dealloc];
}

@end
