/*
 * BOX OFFICES for iPhone/iPad
 *
 * The Original Game is Designed by Entatonic.net
 * Copyright (c) 2010 Entatonic.net
 * http://entatonic.net
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */



#import "StageManager.h"
#import "iPhone_iPad_Selector.h"

#define NUM_OF_STAGE_ROUND 8
#define CEILING_DOWN_LIMITTER 15

@implementation StageManager

-(id) init {
	self = [super init];
	if(self != nil){
		stage_loop = 1;
		stage_round = 1;
		stage_num = 1;
		log_stage_loop = 1;
		log_stage_round = 1;
		log_stage_num = 1;
		launcher_index = 0;
		[self loadStageData];
		[self loadGameData];
	}
	return self;
}

-(NSArray *) getStageData{
	return self.m_stageData;
}

-(void) loadStageData{
	// Load XML Stage Data
	NSString* path = [NSString stringWithFormat: @"%@",
					  [[NSBundle mainBundle] pathForResource:
					   [NSString stringWithFormat:@"StageData"]
													  ofType:@"xml"]];
	if([[NSFileManager defaultManager] fileExistsAtPath:path]){
		NSData* data;
		data = [NSData dataWithContentsOfFile:path];
		
		NSString* string;
		string = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
		self.m_stageData = [string propertyList];
	}

}

-(void) resetStage{
	stage_loop = 1;
	stage_round = 1;
	stage_num = 1;

	launcher_index = 0;
}
-(void) continueStage{
	stage_loop = log_stage_loop;
	stage_round = log_stage_round;
	stage_num = log_stage_num;
	launcher_index = 0;
}

-(BOOL) incrementStage{

	stage_num++;
	launcher_index = 0;
	
	if(stage_num > 4 && stage_round == 0){		// Go to Next round for tutorial
		stage_num = 1;
		stage_round++;		
		if(stage_round > NUM_OF_STAGE_ROUND){
			stage_loop++;
			stage_round = 1;
		}
		return TRUE;
	}

	BOOL Roundup = FALSE;
	if(stage_round != 0){
		if(stage_num > 4){						// Go to Next round for game
			stage_num = 1;
			stage_round++;		

			if(stage_round > NUM_OF_STAGE_ROUND){
				stage_loop++;
				stage_round = 1;
			}
			Roundup = TRUE;
		}
		log_stage_num = stage_num;
		log_stage_round = stage_round;		
		log_stage_loop = stage_loop;
		
		[self saveGameData];	// ver 1.1 Auto Save
	}
	
	return Roundup;
}

-(int) getStageLoop{
	return stage_loop;
}

-(int) getLogStageLoop{
	return log_stage_loop;
}

-(int) getLogStageNum{
	return log_stage_num;
}

-(int) getStageRound{
	return stage_round;
}

-(int) getLogStageRound{
	return log_stage_round;
}
-(int) getStageNum{
	return stage_num;
}



-(BOOL) IsTutorialStage{
	if( stage_round == 0)
		return TRUE;
	else
		return FALSE;
}



// Get Stage Data
-(NSString *) getBoxImage:(int) index{		// index 1-4
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *boxImagePathArray = [stageDataDictionary objectForKey:@"box_image_path"];
	
	if(iPhone_iPad == iPhone){
		return [NSString stringWithFormat:@"%@_iPhone.png", [boxImagePathArray objectAtIndex:index-1]];	// iPhone (1)
	}else{
		return [NSString stringWithFormat:@"%@.png", [boxImagePathArray objectAtIndex:index-1]];		// iPad (2)
	}
}

-(int) getBoxImageCount{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *boxImagePathArray = [stageDataDictionary objectForKey:@"box_image_path"];
	return [boxImagePathArray count];
}

-(NSString *) getBackGournd_image{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];

	
	if(iPhone_iPad == iPhone){
		return [NSString stringWithFormat:@"%@_iPhone.jpg", [stageDataDictionary objectForKey:@"background_imagepath"]];	// iPhone (1)
	}else{
		return [NSString stringWithFormat:@"%@.jpg",		[stageDataDictionary objectForKey:@"background_imagepath"]];	// iPad (2)
	}
}

-(NSString *) getBgm{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	
	if(iPhone_iPad == iPhone){
		return [NSString stringWithFormat:@"%@_iPhone", [stageDataDictionary objectForKey:@"bgm_path"]];	// iPhone (1)
	}else{
		return [NSString stringWithFormat:@"%@",		[stageDataDictionary objectForKey:@"bgm_path"]];	// iPad (2)
	}	
}
-(NSString *) getBgm_song_name{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	return [NSString stringWithFormat:@"%@", [stageDataDictionary objectForKey:@"bgm_song_name"]];
}
-(NSString *) getBgm_artist_name{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	return [NSString stringWithFormat:@"%@", [stageDataDictionary objectForKey:@"bgm_artist_name"]];
}

-(int) getBgm_artist_name_position_x{
	CGSize s = [[CCDirector sharedDirector] winSize];
	
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	if( [[stageDataDictionary objectForKey:@"bgm_artist_name_position"] compare:@"bottom_left"] ==NSOrderedSame){
		return s.width / 5;	// left
	}
	if( [[stageDataDictionary objectForKey:@"bgm_artist_name_position"] compare:@"upper_center"] ==NSOrderedSame){
		return s.width / 2;	// Center
	}
	return s.width / 5;	// default bottom left
}

-(int) getBgm_artist_name_position_y{
	CGSize s = [[CCDirector sharedDirector] winSize];
	
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	if( [[stageDataDictionary objectForKey:@"bgm_artist_name_position"] compare:@"bottom_left"] ==NSOrderedSame){
		return s.height / 9;	// bottom
	}
	if( [[stageDataDictionary objectForKey:@"bgm_artist_name_position"] compare:@"upper_center"] ==NSOrderedSame){
		return s.height * 6 / 7;	// Uptter
	}
	return s.height / 9;	// default bottom left
}

-(float) getFriction{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	return [[DataDictionary objectForKey:@"Friction"] floatValue];
}

-(int) countInitialBoxes{
	NSArray * InitialBoxArray = [self getInitialBoxArray];
	return [InitialBoxArray count];
}
-(float) get_initialBox_x:(int) index{
	NSArray * InitialBoxArray = [self getInitialBoxArray];
	NSDictionary *InitialBox = [InitialBoxArray objectAtIndex:index];
	return [[InitialBox objectForKey:@"x"] floatValue];
}
-(float) get_initialBox_y:(int) index{
	NSArray * InitialBoxArray = [self getInitialBoxArray];
	NSDictionary *InitialBox = [InitialBoxArray objectAtIndex:index];
	return [[InitialBox objectForKey:@"y"] floatValue];
}
-(int) get_initialBox_vx:(int) index{
	NSArray * InitialBoxArray = [self getInitialBoxArray];
	NSDictionary *InitialBox = [InitialBoxArray objectAtIndex:index];
	return [[InitialBox objectForKey:@"vx"] intValue];
}
-(int) get_initialBox_vy:(int) index{
	NSArray * InitialBoxArray = [self getInitialBoxArray];
	NSDictionary *InitialBox = [InitialBoxArray objectAtIndex:index];
	return [[InitialBox objectForKey:@"vy"] intValue];
}
-(int) get_initialBox_type:(int) index{
	NSArray * InitialBoxArray = [self getInitialBoxArray];
	NSDictionary *InitialBox = [InitialBoxArray objectAtIndex:index];
	return [[InitialBox objectForKey:@"type"] intValue];
}
-(int) get_initialBox_scale:(int) index{
	NSArray * InitialBoxArray = [self getInitialBoxArray];
	NSDictionary *InitialBox = [InitialBoxArray objectAtIndex:index];
	return [[InitialBox objectForKey:@"scale"] intValue];
}
-(NSArray *) getInitialBoxArray{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	return [DataDictionary objectForKey:@"Boxes"];
}

-(int) get_launcherType{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	if( [[DataDictionary objectForKey:@"LauncherType"] compare:@"random"] ==NSOrderedSame){
		return random;
	}
	if( [[DataDictionary objectForKey:@"LauncherType"] compare:@"loop"] ==NSOrderedSame){
		return loop;
	}
	return random;	// default
}
-(int) get_launcherTypeisLoop_Count{						// count of loop, if launcher type is loop
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	return [[DataDictionary objectForKey:@"launcherLoop"] count];
}
-(int) get_launcher_NextboxType{			// get type, if launcher type is loop
	if([self get_launcherType] == random){
		return rand() % [self getBoxImageCount] + 1;
	}
	if([self get_launcherType] == loop){
		NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
		NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
		NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
		NSArray *launcherLoopDataArray = [DataDictionary objectForKey:@"launcherLoop"];
	
		int index = launcher_index;
		launcher_index++;
		if ([self get_launcherTypeisLoop_Count] <= launcher_index) launcher_index = 0;
	
		return [[launcherLoopDataArray objectAtIndex:index] intValue];
	}
	return 0;
}


// Text Color

-(float) getTextShadowColor:(int) index{	// 0: R, 1: G, 2: B
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *textShadowColorArray = [stageDataDictionary objectForKey:@"text_shadow_color"];
	
	return [[textShadowColorArray objectAtIndex:index] floatValue];
}

-(float) getFieldLineColor:(int) index{	// 0: R, 1: G, 2: B, 3
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	NSArray *fieldColorDataArray = [DataDictionary objectForKey:@"FieldLineColor"];
	
	return [[fieldColorDataArray objectAtIndex:index] floatValue];
}
-(float) getStageStringColor:(int) index{	// 0: R, 1: G, 2: B
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	NSArray *fieldColorDataArray = [DataDictionary objectForKey:@"StageStringColor"];
	
	return [[fieldColorDataArray objectAtIndex:index] floatValue];
}
-(float) getArtistNameStringColor:(int) index{	// 0: R, 1: G, 2: B
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	NSArray *fieldColorDataArray = [DataDictionary objectForKey:@"ArtistNameStringColor"];
	
	return [[fieldColorDataArray objectAtIndex:index] floatValue];
}
-(float) getClearStringColor:(int) index{	// 0: R, 1: G, 2: B
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	NSArray *fieldColorDataArray = [DataDictionary objectForKey:@"ClearStringColor"];
	
	return [[fieldColorDataArray objectAtIndex:index] floatValue];
}
-(float) getGameOverStringColor:(int) index{	// 0: R, 1: G, 2: B
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	NSArray *fieldColorDataArray = [DataDictionary objectForKey:@"GameOverStringColor"];
	
	return [[fieldColorDataArray objectAtIndex:index] floatValue];
}
-(int) getCielingDown{
	NSDictionary *stageDataDictionary = [self.m_stageData objectAtIndex:stage_round];
	NSArray *stageDataArray = [stageDataDictionary objectForKey:@"StageData"];
	NSDictionary *DataDictionary = [stageDataArray objectAtIndex:stage_num-1];
	
	int Ceiling_down = [[DataDictionary objectForKey:@"CielingDown"] intValue];
	Ceiling_down = Ceiling_down + (stage_loop - 1) * 4;
	if(Ceiling_down > CEILING_DOWN_LIMITTER) Ceiling_down = CEILING_DOWN_LIMITTER;
	
	return Ceiling_down * iPhone_iPad;
}


// ver 1.1 auto save function
- (void) loadGameData{
	NSArray* paths;
	NSString* path;
	paths = NSSearchPathForDirectoriesInDomains(
												NSDocumentDirectory, NSUserDomainMask, YES);
	path = [paths objectAtIndex:0];
	path = [path stringByAppendingPathComponent:@"GameData"];
	
	if([[NSFileManager defaultManager] fileExistsAtPath:path]){	// load if the file exists
		NSData* data;
		data = [NSData dataWithContentsOfFile:path];
		
		NSString* string;
		string = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
		
		NSMutableArray *GameData;
		GameData = [string propertyList];
		int FormatVersion		= [[GameData objectAtIndex:0] intValue];
		if(FormatVersion == 1){	// Never Change here in the future to keep data compatibility
			log_stage_loop			= [[GameData objectAtIndex:1] intValue];
			log_stage_round			= [[GameData objectAtIndex:2] intValue];
			log_stage_num			= [[GameData objectAtIndex:3] intValue];
		}
		
	}else{														// init
		log_stage_loop = 1;
		log_stage_round = 1;
		log_stage_num = 1;
		[self saveGameData];
	}
}

- (void) saveGameData{
	NSArray* paths;
	NSString* path;
	paths = NSSearchPathForDirectoriesInDomains(
												NSDocumentDirectory, NSUserDomainMask, YES);
	path = [paths objectAtIndex:0];
	path = [path stringByAppendingPathComponent:@"GameData"];
	
	NSMutableArray *GameData;
	GameData = [[NSMutableArray alloc] init];
	[GameData insertObject:[NSNumber numberWithInt:1] atIndex:0];				// Save format version
	[GameData insertObject:[NSNumber numberWithInt:log_stage_loop] atIndex:1];		// stage_loop
	[GameData insertObject:[NSNumber numberWithInt:log_stage_round] atIndex:2];		// stage_round
	[GameData insertObject:[NSNumber numberWithInt:log_stage_num] atIndex:3];		// stage_num
		  
	[[GameData description] writeToFile:path atomically:YES];	
}

-(void) dealloc
{
	[super dealloc];
}

@synthesize stage_round;
@synthesize stage_num;
@synthesize m_stageData;

@end
